"""Interval Model. Useful to limit in a time frame some tools"""

from pollenisatorgui.core.components.datamanager import DataManager
from pollenisatorgui.core.models.element import Element
from pollenisatorgui.core.models.tool import Tool
from pollenisatorgui.core.components.apiclient import APIClient
import pollenisatorgui.core.components.utils as utils
from bson.objectid import ObjectId
from datetime import datetime


class Interval(Element):
    """
    Represents an interval object that defines an time interval where a wave can be executed.

    Attributes:
        coll_name: collection name in pollenisator database
    """
    coll_name = "intervals"

    def __init__(self, valuesFromDb=None):
        """Constructor
        Args:
            valueFromDb: a dict holding values to load into the object. A mongo fetched interval is optimal.
                        possible keys with default values are : _id (None), parent (None), tags([]), infos({}),
                        wave(""), dated("None"), datef("None")
        """
        if valuesFromDb is None:
            valuesFromDb = {}
        super().__init__(valuesFromDb.get("_id", None), valuesFromDb.get("parent", None),  valuesFromDb.get("infos", {}))
        self.initialize(valuesFromDb.get("wave", ""), valuesFromDb.get("dated", "None"),
                        valuesFromDb.get("datef", "None"), valuesFromDb.get("infos", {}))

    def initialize(self, wave, dated="None", datef="None", infos=None):
        """Set values of interval
        Args:
            wave: the parent wave name
            dated: a starting date and tiem for this interval in format : '%d/%m/%Y %H:%M:%S'. or the string "None"
            datef: an ending date and tiem for this interval in format : '%d/%m/%Y %H:%M:%S'. or the string "None"
            infos: a dictionnary with key values as additional information. Default to None
        Returns:
            this object
        """
        self.wave = wave
        self.dated = dated
        self.datef = datef
        self.infos = infos if infos is not None else {}
        return self

    def delete(self):
        """
        Delete the Interval represented by this model in database.
        """
        apiclient = APIClient.getInstance()
        apiclient.delete(
            "intervals", self._id)
        

    def addInDb(self):
        """
        Add this interval in database.

        Returns: a tuple with :
                * bool for success
                * mongo ObjectId : already existing object if duplicate, create object id otherwise 
        """
        base = {"wave": self.wave, "dated": self.dated, "datef": self.datef}
        apiclient = APIClient.getInstance()
        res, iid = apiclient.insert("intervals", base)
        self._id = iid
        return True, iid

    def update(self, pipeline_set=None):
        """Update this object in database.
        Args:
            pipeline_set: (Opt.) A dictionnary with custom values. If None (default) use model attributes.
        """
        
        apiclient = APIClient.getInstance()
        if pipeline_set is None:
            apiclient.update("intervals", ObjectId(self._id), {"dated": self.dated, "datef": self.datef})
        else:
            apiclient.update("intervals", ObjectId(self._id), pipeline_set)

    def _getParentId(self):
        """
        Return the mongo ObjectId _id of the first parent of this object. For an interval it is the wave.

        Returns:
            Returns the parent wave's ObjectId _id".
        """
        datamanager = DataManager.getInstance()
        obj = datamanager.find("waves", {"wave": self.wave}, False)
        if obj is None:
            return None
        return obj["_id"]

    def getData(self):
        """Return interval attributes as a dictionnary matching Mongo stored intervals
        Returns:
            dict with keys wave, dated, datef, _id, tags and infos
        """
        return {"wave": self.wave, "dated": self.dated, "datef": self.datef, "_id": self.getId(), "infos": self.infos}


    def __str__(self):
        """
        Get a string representation of an interval

        Returns:
            Returns the string "Interval".
        """
        return "Interval"

    @classmethod
    def _translateDateString(cls, datestring):
        """Returns the datetime object when given a str wih format '%d/%m/%Y %H:%M:%S'
        Args:
            a string formated as datetime format : '%d/%m/%Y %H:%M:%S'
        """
        ret = None
        if(type(datestring) == str or type(datestring) == str):
            if datestring != "None":
                ret = datetime.strptime(
                    datestring, '%d/%m/%Y %H:%M:%S')
        return ret

    def getEndingDate(self):
        """Returns the ending date and time of this interval
        Returns:
            a datetime object.
        """
        return Interval._translateDateString(self.datef)
    
    def getStartingDate(self):
        """Returns the starting date and time of this interval
        Returns:
            a datetime object.
        """
        return Interval._translateDateString(self.dated)

    def getDbKey(self):
        """Return a dict from model to use as unique composed key.
        Returns:
            A dict (1 key :"wave")
        """
        return {"wave": self.wave}



from customtkinter import CTkEntry

class PopoEntry(CTkEntry):
    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)
        self.bind("<Control-a>", self.onSelectAll)

    def onSelectAll(self, _event=None):
        # select text
        self.select_range(0, 'end')
        # move cursor to the end
        self.icursor('end')
        return "break"


"""View for checkitem object. Handle node in treeview and present forms to user when interacted with."""

import tkinter.ttk as ttk
import webbrowser
from customtkinter import *
from pollenisatorgui.core.application.dialogs.ChildDialogDefectView import ChildDialogDefectView
from pollenisatorgui.core.application.dialogs.ChildDialogProgress import ChildDialogProgress
from pollenisatorgui.core.application.dialogs.ChildDialogToast import ChildDialogToast
from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.core.components.datamanager import DataManager
from pollenisatorgui.core.components.scriptmanager import ScriptManager
from pollenisatorgui.core.components.tag import TagInfos
from pollenisatorgui.core.controllers.checkinstancecontroller import CheckInstanceController
from pollenisatorgui.core.controllers.defectcontroller import DefectController
from pollenisatorgui.core.controllers.toolcontroller import ToolController
from pollenisatorgui.core.models.defect import Defect
from pollenisatorgui.core.views.checkinstancemultiview import CheckInstanceMultiView
from pollenisatorgui.core.views.defectview import DefectView
from pollenisatorgui.core.views.toolview import ToolView
from pollenisatorgui.core.views.viewelement import ViewElement
from pollenisatorgui.core.components.settings import Settings
from pollenisatorgui.core.models.tool import Tool
import pollenisatorgui.core.components.utils as utils
from PIL import ImageTk, Image
from pollenisatorgui.core.components.logger_config import logger
from bson import ObjectId
import os
import tkinter as tk


class CheckInstanceView(ViewElement):
    """
    View for CheckInstanceView object. Handle node in treeview and present forms to user when interacted with.
    Attributes:
        icon: icon name to show in treeview. Icon filename must be in icon directory.
    """
    done_icon = 'checked.png'
    running_icon = 'running.png'
    not_done_icon = 'unchecked.png'
    icon = 'checklist.png'

    cached_icon = None
    cached_done_icon = None
    cached_running_icon = None
    cached_not_ready_icon = None

    multiview_class = CheckInstanceMultiView

    @classmethod
    def getStatusIcon(cls, status):
        if "todo" in status: # order is important as "done" is not_done but not the other way
            ui = cls.not_done_icon
            cache = cls.cached_not_ready_icon
            iconStatus = "not_done"
        elif "running" in status:
            ui = cls.running_icon
            cache = cls.cached_running_icon
            iconStatus = "running"
        else:
            cache = cls.cached_done_icon
            ui = cls.done_icon
            iconStatus = "done"
        if cache is None:
            from PIL import Image, ImageTk
            if iconStatus == "done":
                cls.cached_done_icon = utils.loadIcon(ui, resize=(16,16))
                return cls.cached_done_icon
            elif iconStatus == "running":
                cls.cached_running_icon =  utils.loadIcon(ui, resize=(16,16))
                return cls.cached_running_icon
            else:
                cls.cached_not_ready_icon =  utils.loadIcon(ui, resize=(16,16))
                return cls.cached_not_ready_icon
        return cache

    def getIcon(self, check_infos=None):
        """
        Load the object icon in cache if it is not yet done, and returns it

        Return:
            Returns the icon representing this object.
        """
        
        status = self.controller.getStatus()
        iconStatus = "todo"
        if "todo" in status or "" == status: # order is important as "done" is not_done but not the other way
            ui = self.__class__.not_done_icon
            cache = self.__class__.cached_not_ready_icon
            iconStatus = "not_done"
        elif "running" in status:
            ui = self.__class__.running_icon
            cache = self.__class__.cached_running_icon
            iconStatus = "running"
        elif "done" in status:
            cache = self.__class__.cached_done_icon
            ui = self.__class__.done_icon
            iconStatus = "done"
        # FIXME, always get triggered
        # if status == [] or iconStatus not in status :
        #     print("status is "+str(status)+ " or "+str(iconStatus)+" not in "+str(status))
        #     self.controller.setStatus([iconStatus])

        if cache is None:
            from PIL import Image, ImageTk
            if iconStatus == "done":
                self.__class__.cached_done_icon = utils.loadIcon(ui, resize=(16,16))
                return self.__class__.cached_done_icon
            elif iconStatus == "running":
                self.__class__.cached_running_icon =  utils.loadIcon(ui, resize=(16,16))
                return self.__class__.cached_running_icon
            else:
                self.__class__.cached_not_ready_icon =  utils.loadIcon(ui, resize=(16,16))
                return self.__class__.cached_not_ready_icon
        return cache

    def getStatus(self, check_infos=None):
        modelData = self.controller.getData()
        if modelData.get("status", "") == "done":
            status = "done"
        else:
            if check_infos is None:
                check_infos = self.controller.getCheckInstanceStatus()
            status = check_infos.get("status", "")
        if status == "":
            status = modelData.get("status", "")
        if status == "":
            status = "todo"
        return status

    def __init__(self, appTw, appViewFrame, mainApp, controller):
        """Constructor
        Args:
            appTw: a PollenisatorTreeview instance to put this view in
            appViewFrame: an view frame to build the forms in.
            mainApp: the Application instance
            controller: a CommandController for this view.
        """
        self.menuContextuel = None
        self.widgetMenuOpen = None
        
        super().__init__(appTw, appViewFrame, mainApp, controller)

    
            
    def openModifyWindow(self, **kwargs):
        """
        Creates a tkinter form using Forms classes. This form aims to update or delete an existing Command
        """
       
        self.form.clear()
        infos = self.controller.getCheckInstanceInfo()
        modelData = self.controller.getData()
        check_m = self.controller.getCheckItem()
        self._initContextualMenu()
        self.form.addFormHidden("parent", modelData.get("parent"))
        panel_top = self.form.addFormPanel(grid=True)
        
        
        #panet_top_sub = panel_top.addFormPanel(grid=True, row=3, column=0, columnspan=2)
        panel_top.addFormLabel("Target", row=0, column=0)
        panel_top.addFormButton(self.controller.target_repr, self.openTargetDialog, row=0, column=1, style="link.TButton", pady=5)
        panel_top.addFormLabel("Status", row=0, column=2)
        default_status = self.getStatus(infos)
        self.image_terminal = CTkImage(Image.open(utils.getIcon('terminal_small.png')))
        self.form_status = panel_top.addFormCombo("Status", ["todo", "running","done"], default=default_status, command=self.status_change, row=0, column=3, pady=5)
        #panet_top_sub.addFormButton("Attack", callback=self.attackOnTerminal, image=self.image_terminal, row=0, column=2)
        
        #if "commands" in check_m.check_type:

        self.buttonExecuteImage = CTkImage(Image.open(utils.getIcon('execute.png')))
        self.buttonQueueImage = CTkImage(Image.open(utils.getIcon('exec_cloud.png')))
        self.buttonRunImage = CTkImage(Image.open(utils.getIcon('terminal_small.png')))
        self.buttonDownloadImage = CTkImage(Image.open(utils.getIcon("download.png")))
        self.image_download = utils.loadIcon("download.png")
        self.image_reset = utils.loadIcon("reset_small.png")
        self.image_delete = utils.loadIcon("delete.png")
        self.image_defect = utils.loadIcon("defect.png")
        dict_of_tools_not_done = infos.get("tools_not_done", {})
        dict_of_tools_error = infos.get("tools_error", {})
        dict_of_tools_running = infos.get("tools_running", {})
        dict_of_tools_done = infos.get("tools_done", {})
        #create lambdas indirection for each button
        lambdas_lauch_tool_local = [self.launchToolLocalCallbackLambda(iid) for iid in dict_of_tools_not_done.keys()]
        lambdas_queue_tool_worker = [self.queueToolCallbackLambda(iid) for iid in dict_of_tools_not_done.keys()]
        lambdas_launch_tool_worker = [self.launchToolWorkerCallbackLambda(iid) for iid in dict_of_tools_not_done.keys()]
        lambdas_running = [self.peekToolCallbackLambda(iid) for iid in dict_of_tools_running.keys()]
        lambdas_running_stop = [self.stopToolCallbackLambda(iid) for iid in dict_of_tools_running.keys()]
        lambdas_done = [self.downloadToolCallbackLambda(iid) for iid in dict_of_tools_done.keys()]
        lambdas_del = [self.deleteToolCallbackLambda(iid) for iid in dict_of_tools_done.keys()]
        lambdas_error_reset = [self.resetToolCallbackLambda(iid) for iid in dict_of_tools_error.keys()]
        lambdas_defect_create = [self.createDefectCallbackLambda(iid) for iid in dict_of_tools_done.keys()]
        lambdas_reset = [self.resetToolCallbackLambda(iid) for iid in dict_of_tools_done.keys()]
        datamanager = DataManager.getInstance()
        apiclient = APIClient.getInstance()
        
        if dict_of_tools_not_done:
            self.form.addFormSeparator()
            self.form.addFormLabel("Command suggestions", font_size=20, side=tk.TOP, anchor=tk.W)
            formCommands = self.form.addFormPanel(side=tk.TOP, fill=tk.X, pady=5, grid=True)
            row=0
            for tool_iid, tool_string in dict_of_tools_not_done.items():
                success, data = apiclient.getCommandLine(tool_iid)
                if success:
                    comm, fileext = data["comm"], data["ext"]
                    toolModel =  Tool.fetchObject({"_id":ObjectId(tool_iid)})
                    if toolModel is None:
                        continue
                    formCommands.addFormButton(toolModel.name, self.openToolDialog, row=row, column=0, style="link.TButton", infos={"iid":tool_iid})
                    form_str = None
                    if success:
                        commandModel = datamanager.get("commands", toolModel.command_iid)
                        form_str_bin= formCommands.addFormStr("bin_path", "", commandModel.bin_path, status="disabled", width=80, row=row, column=1)
                        form_str= formCommands.addFormStr("commandline", "", comm, width=550, row=row, column=2)

                    formCommands.addFormButton("Execute", lambdas_lauch_tool_local[row], row=row, column=3, width=0, infos= {'formstr': form_str, "bin":form_str_bin}, image=self.buttonExecuteImage)
                    ready, msg = self.mainApp.scanManager.is_ready_to_queue(str(tool_iid))
                    if ready:
                        formCommands.addFormButton("Queue", lambdas_queue_tool_worker[row], row=row, column=4, width=0, infos= {'formstr': form_str, "bin":form_str_bin}, 
                                            state="normal" if ready else "disabled", image=self.buttonQueueImage)
                    ready, msg = self.mainApp.scanManager.is_ready_to_run_tasks()
                    if ready:
                        formCommands.addFormButton("Worker execute", lambdas_launch_tool_worker[row], row=row, column=5, width=0, infos= {'formstr': form_str, "bin":form_str_bin}, 
                                            state="normal" if ready else "disabled", image=self.buttonQueueImage)
                row+=1
        if dict_of_tools_running:
            formCommands = self.form.addFormPanel(side=tk.TOP, fill=tk.X, pady=5, grid=True)
            row=0
            for tool_iid, tool_string in dict_of_tools_running.items():
                formCommands.addFormSeparator(row=row, column=0, columnspan=3)
                formCommands.addFormButton(tool_string, self.openToolDialog, row=row*2+1, column=0,style="link.TButton", infos={"iid":tool_iid})
                tool_model = datamanager.get("tools", tool_iid)
                form_str = formCommands.addFormStr("commandline", "", tool_model.text, status="disabled", width=550, row=row*2+1, column=2)
                formCommands.addFormButton("Peek", lambdas_running[row], row=row*2+1, column=3, width=0, image=self.buttonRunImage)
                formCommands.addFormButton("Stop", lambdas_running_stop[row], row=row*2+1, column=4,  width=0, fg_color=utils.getBackgroundColor(), text_color=utils.getTextColor(),
                               border_width=1, border_color="firebrick1", hover_color="tomato")
                row+=1
        if dict_of_tools_done:
            self.form.addFormSeparator()
            self.form.addFormLabel("Results", font_size=20, side=tk.TOP, anchor=tk.CENTER)
            formCommands = self.form.addFormPanel(side=tk.TOP, fill=tk.X, pady=5)
            row=0
            for tool_iid, tool_data in dict_of_tools_done.items():
                tool_m = Tool(tool_data)
                tool_panel = formCommands.addFormPanel(side=tk.TOP, fill=tk.X, pady=0)
                #tool_panel.addFormSeparator(fill=tk.X)
                tool_panel.addFormButton(tool_m.getDetailedString(), self.openToolDialog, side=tk.TOP, anchor=tk.W, style="link.TButton", infos={"iid":tool_iid})
                tags = ToolController(tool_m).getTags()
                if tags:
                    for tag in tags:
                        tag = TagInfos(tag)
                        registeredTags = Settings.getTags()
                        keys = list(registeredTags.keys())
                        column = 0
                        item_no = 0
                        
                        s = ttk.Style(self.mainApp)
                        if tag.color is None or tag.color == "transparent":
                            tag_color = registeredTags.get(tag.name, {}).get("color"," gray97")
                        else:
                            tag_color = tag.color
                        try: # CHECK IF COLOR IS VALID
                            CTkLabel(self.mainApp, fg_color=tag_color)
                        except tk.TclError as e:
                            #color incorrect
                            tag_color = "gray97"
                        s.configure(""+tag_color+".Default.TLabel", background=tag_color, foreground="black", borderwidth=1, bordercolor="black")
                        tool_panel.addFormLabel(tag, text=tag.name, side="top", padx=1, pady=0)
                        column += 1
                        item_no += 1
                        if column == 4:
                            column = 0
                        if column == 0:
                            tool_panel = tool_panel.addFormPanel(pady=0,side=tk.TOP, anchor=tk.W)
                tool_panel.addFormText(str(tool_iid)+"_notes", "", default=tool_m.notes,  side=tk.LEFT, height=min(26*+len(tool_m.notes.split("\n")), 200))
                action_panel = tool_panel.addFormPanel(side=tk.LEFT, anchor=tk.CENTER, fill=tk.BOTH, grid=True)
                action_panel.rowconfigure(0, 2)
                action_panel.addFormButton("", lambdas_done[row], image=self.image_download, style="icon.TButton", tooltip="Download original result file generated by the tool")
                action_panel.addFormButton("", lambdas_defect_create[row],  image=self.image_defect, style="icon.TButton", tooltip="Create a security defect instance with this tool notes", column=1)
                action_panel.addFormButton("", lambdas_reset[row], image=self.image_reset, style="icon.TButton", tooltip="Reset the tool data. All data will be lost.", column=2)
                action_panel.addFormButton("", lambdas_del[row], image=self.image_delete, style="icon.TButton", tooltip="Delete the tool data. Tool results too.", column=3)
                row+=1
        if dict_of_tools_error:
            formCommands = self.form.addFormPanel(side=tk.TOP, fill=tk.X, pady=5, grid=True)
            row=0
            formCommands.addFormSeparator(row=row, column=0, columnspan=3)
            for tool_iid, tool_data in dict_of_tools_error.items():
                tool_m = Tool(tool_data)
                formCommands.addFormButton(tool_m.getDetailedString(), self.openToolDialog, row=row,column=0,  style="link.TButton", infos={"iid":tool_iid})
                
                formCommands.addFormText(str(tool_iid)+"_notes", "", default=tool_m.notes,  row=row, column=1 , height=min(26*+len(tool_m.notes.split("\n")), 200))
                formCommands.addFormButton("Reset", lambdas_error_reset[row], row=row, column=2,  width=0, fg_color=utils.getBackgroundColor(), text_color=utils.getTextColor(),
                               border_width=1, border_color="firebrick1", hover_color="tomato")
                row+=1
        
        upload_panel = self.form.addFormPanel(side=tk.TOP, fill=tk.X,pady=5, height=0)
        upload_panel.addFormLabel("Upload additional scan results", side=tk.LEFT, anchor=tk.N, pady=5)
        self.form_file = upload_panel.addFormFile("upload_tools", height=2, side=tk.LEFT, pady=5, command=self.mod_file_callback)
        self.upload_btn = upload_panel.addFormButton("Upload", callback=self.upload_scan_files, state="disabled", anchor=tk.N, side=tk.LEFT, pady=5)

        #for command, status in infos.get("tools_status", {}).items():
        if check_m and "script" in check_m.check_type:
            formTv = self.form.addFormPanel(side=tk.TOP, fill=tk.X, pady=5)
            formTv.addFormLabel("Script", side=tk.LEFT)
            self.textForm = formTv.addFormStr("Script", ".+", check_m.script)
            self.execute_icon = CTkImage(Image.open(utils.getIcon("execute.png")))
            self.edit_icon = CTkImage(Image.open(utils.getIcon("view_doc.png")))
            formTv.addFormButton("View", lambda _event: self.viewScript(check_m.script), image=self.edit_icon)
            formTv.addFormButton("Exec", lambda _event: self.execScript(check_m.script), image=self.execute_icon)
        self.defect_titles = {}
        for defect in check_m.defect_tags:
            d = apiclient.findInDb("pollenisator", "defects", {"_id":ObjectId(defect[1])}, False)
            if d is not None:
                if d.get("language", "") == self.mainApp.settings.db_settings.get("lang", "en"):
                    self.defect_titles[d.get("title")] = d
        self.form.addFormSeparator(fill=tk.X, pady=5)
        formDefects = self.form.addFormPanel(fill=tk.X, pady=5, grid=True)
        formDefects.addFormLabel("Add Security defects", pady=5,row=1)
        self.combo_defect = formDefects.addFormCombo("Defect", list(self.defect_titles.keys())+["Search other defect"], default=None, width=300, row=1, column=1)
        formDefects.addFormButton("Confirm", callback=self.defect_add_callback, row=1, column=2)
        self.form.addFormSeparator(fill=tk.X, pady=5)
        panel_detail = self.form.addFormPanel(grid=True, fill=tk.X)
        panel_detail.columnconfigure(1, weight=2)
        panel_detail.addFormLabel("Description", row=1, column=0)
        panel_detail.addFormText("Description", r"", default=check_m.description if check_m and check_m.description != "" else "No description", height=100, state="disabled", row=1, column=1, pady=5)
        panel_detail.addFormLabel("Notes", row=2, column=0)
        panel_detail.addFormText("Notes", r"", default=modelData.get("notes", ""), row=2, column=1, pady=5)
        self.completeModifyWindow(addTags=False)
        
    def mod_file_callback(self):
        if self.form_file.get_paths():
            self.upload_btn.configure(state="normal")
        else:
            self.upload_btn.configure(state="disabled")
    
    def attackOnTerminal(self, event=None):
        #import pollenisatorgui.modules.terminal as terminal
        #terminal.Terminal.openTerminal(str(self.controller.getDbId()))
        self.mainApp.open_terminal(str(self.controller.getDbId()), self.controller.target_repr)

    def defect_add_callback(self, event=None):
        defect_title = self.combo_defect.getValue()
        if defect_title == "Search other defect":
            d = Defect({"target_type":self.controller.model.target_type, "target_id":self.controller.model.target_iid})
            ChildDialogDefectView(self.mainApp, "Search defect", self.mainApp.settings, force_insert=True)
        else:
            defect = self.defect_titles.get(defect_title)
            if defect is not None:
                defect["target_type"] = self.controller.model.target_type
                defect["target_id"] = self.controller.model.target_iid
                
                dv = DefectView(self.appliTw, self.appliViewFrame, self.mainApp,
                                DefectController(Defect(defect)))
                dv.openInsertWindow(defect.get("notes", ""))

    def status_change(self, event):
        status = self.form_status.getValue()
        self.controller.doUpdate({"Status": status})
        caller = self.form_status.box
        #caller = widget.master
        #caller.update_idletasks()
        toast = ChildDialogToast(self.appliViewFrame, "Done" , x=caller.winfo_rootx(), y=caller.winfo_rooty()+caller.winfo_reqheight(), width=caller.winfo_reqwidth())
        toast.show()


    def upload_scan_files(self, event):
        files_paths = self.form_file.getValue()
        files = set()
        for filepath in files_paths:
            if os.path.isdir(filepath):
                # r=root, d=directories, f = files
                for r, _d, f in os.walk(filepath):
                    for fil in f:
                        files.add(os.path.join(r, fil))
            else:
                files.add(filepath)
        if not files:
            return
        dialog = ChildDialogProgress(self.mainApp, "Importing files", "Importing "+str(
            len(files)) + " files. Please wait for a few seconds.", 1/len(files)*50, "determinate")
        dialog.show(len(files))
        # LOOP ON FOLDER FILES
        results = {}
        apiclient = APIClient.getInstance()
        for f_i, file_path in enumerate(files):
            additional_results = apiclient.importExistingResultFile(file_path, "auto-detect", {"check_iid":str(self.controller.getDbId()), "lvl":"import"}, "")
            for key, val in additional_results.items():
                results[key] = results.get(key, 0) + val
            dialog.update()
        dialog.destroy()
        # DISPLAY RESULTS
        presResults = ""
        filesIgnored = 0
        for key, value in results.items():
            presResults += str(value) + " " + str(key)+".\n"
            if key == "Ignored":
                filesIgnored += 1
        if filesIgnored > 0:
            tk.messagebox.showwarning(
                "Auto-detect ended", presResults, parent=self.mainApp)
        else:
            tk.messagebox.showinfo("Auto-detect ended", presResults, parent=self.mainApp)
        self.openModifyWindow()

       
    
    def launchToolLocalCallbackLambda(self, tool_iid, **kwargs):
        return lambda event, kwargs: self.launchToolLocalCallback(tool_iid, **kwargs)
    
    def queueToolCallbackLambda(self, tool_iid, **kwargs):
        return lambda event, kwargs: self.queueToolWorkerCallback(tool_iid, **kwargs)

    def launchToolWorkerCallbackLambda(self, tool_iid, **kwargs):
        return lambda event, kwargs: self.launchToolWorkerCallback(tool_iid, **kwargs)

    def peekToolCallbackLambda(self, tool_iid):
        return lambda event=None: self.peekToolCallback(tool_iid)
    
    def stopToolCallbackLambda(self, tool_iid):
        return lambda event=None: self.stopToolCallback(tool_iid)

    def downloadToolCallbackLambda(self, tool_iid):
        return lambda event=None: self.downloadToolCallback(tool_iid)
    
    def deleteToolCallbackLambda(self, tool_iid):
        return lambda event=None: self.deleteToolCallback(tool_iid)
    
    def createDefectCallbackLambda(self, tool_iid):
        return lambda _event=None: self.createDefectToolCallback(tool_iid)
    
    def resetToolCallbackLambda(self, tool_iid):
        return lambda event=None: self.resetToolCallback(tool_iid)
    
    def resetToolCallbackLambda(self, tool_iid):
        return lambda event=None: self.resetToolCallback(tool_iid)
    
    def getAdditionalContextualCommands(self):
        ret = {}
        dataManager = DataManager.getInstance()
        target_m = dataManager.get(self.controller.model.target_type, self.controller.model.target_iid)
        if target_m is not None:
            if hasattr(target_m, "getURL"):
                url = target_m.getURL()
                if url != "":
                    ret["Open In Browser"] = self.openInBrowser
        ret["Attack from terminal"] = self.attackOnTerminal
        return ret
    
    def openInBrowser(self):
        dataManager = DataManager.getInstance()
        target_m = dataManager.get(self.controller.model.target_type, self.controller.model.target_iid)
        if target_m is not None:
            if hasattr(target_m, "getURL"):
                url = target_m.getURL()
                if url != "":
                    webbrowser.open_new_tab(url)

    def openToolDialog(self, event, infos):
        tool_iid = infos.get("iid")
        tool_m = Tool.fetchObject({"_id":ObjectId(tool_iid)})
        tool_vw = ToolView(self.appliTw, None, self.mainApp, ToolController(tool_m))
        tool_vw.openInDialog()
        self.openModifyWindow()

    def onDoubleClick(self):
        self.controller.swapStatus()

    def openTargetDialog(self, event):
        data = self.controller.getData()
        datamanager = DataManager.getInstance()
        ret = datamanager.get(data["target_type"].lower(), data["target_iid"])
        if ret is None:
            tk.messagebox.showerror("Error", "Target not found")
            return
        view = self.mainApp.modelToView(data["target_type"], ret)
        if view:
            if hasattr(view, "openInDialog"): # If it is not a ChildDialog already, open it
                view.openInDialog()
        self.openModifyWindow()

    def launchToolLocalCallback(self, tool_iid, **kwargs):
        form_commandline = kwargs.get("formstr", None)
        command_bin = kwargs.get("bin", None)
        tool_m = Tool.fetchObject({"_id":ObjectId(tool_iid)})

        if form_commandline is not None:
            command_line = form_commandline.getValue()
            tool_m.text = command_line
            tool_m.update({"text":command_line})
            settings = Settings()
            my_commands = settings.local_settings.get("my_commands", {})
            bin_path = my_commands.get(tool_m.name)
            if bin_path is None:
                bin_path = command_bin.getValue()
            bin_path_found = utils.which_expand_alias(bin_path)
            if bin_path_found and bin_path_found != "":
                self.mainApp.launch_tool_in_terminal(tool_m, bin_path_found+" "+command_line)
            else:
                tk.messagebox.showerror("Could not launch this tool", f"Binary path is not available ({bin_path}), is it not installed ?")
            #

    def launchToolWorkerCallback(self, tool_iid, **kwargs):
        apiclient = APIClient.getInstance()
        form_commandline = kwargs.get("formstr", None)
        
        tool_m = Tool.fetchObject({"_id":ObjectId(tool_iid)})
        self.mainApp.subscribe_notification("tool_start", self.toolStartedEvent, pentest=apiclient.getCurrentPentest(), iid=tool_iid)
        if form_commandline is not None:
            command_line = form_commandline.getValue()
            tool_m.text = command_line
            tool_m.update({"text":command_line})
        results = apiclient.runTask(tool_iid)
        if not results:
            self.mainApp.unsubscribe_notification("tool_start", self.toolStartedEvent, pentest=apiclient.getCurrentPentest(), iid=tool_iid)
      

    def queueToolWorkerCallback(self, tool_iid, **kwargs):
        apiclient = APIClient.getInstance()
        form_commandline = kwargs.get("formstr", None)
        
        tool_m = Tool.fetchObject({"_id":ObjectId(tool_iid)})

        if form_commandline is not None:
            command_line = form_commandline.getValue()
            tool_m.text = command_line
            tool_m.update({"text":command_line})
        results = apiclient.sendQueueTasks(tool_iid)
        if results.get("successes", []):
            self.mainApp.subscribe_notification("tool_start", self.toolStartedEvent, pentest=apiclient.getCurrentPentest(), iid=tool_iid)
            #self.mainApp.after(600, self.openModifyWindow)
        else:
            if results.get("failures", []):
                failure = results.get("failures", [])[0]
                tk.messagebox.showerror("Error", "Error while queueing task : "+str(failure.get("error", "unknown error")), parent=self.mainApp)

    def toolStartedEvent(self, notification):
        iid = notification.get("iid")
        if iid is None:
            return
        apiclient = APIClient.getInstance()
        self.mainApp.unsubscribe_notification("tool_start", pentest=apiclient.getCurrentPentest(), iid=iid)
        self.peekToolCallback(iid)

    def peekToolCallback(self, tool_iid):
        tool_controller = ToolController(Tool.fetchObject({"_id":ObjectId(tool_iid)}))
        self.mainApp.open_any_terminal(str(self.controller.getDbId())+"|"+str(tool_controller.getDbId()), tool_controller.getDetailedString(), tool_controller, self.mainApp.scanManager)
        
    def stopToolCallback(self, tool_iid):
        tool_m = Tool.fetchObject({"_id":ObjectId(tool_iid)})
        tool_vw = ToolView(self.appliTw, self.appliViewFrame, self.mainApp, ToolController(tool_m))
        tool_vw.stopCallback()

    def downloadToolCallback(self, tool_iid):
        tool_m = Tool.fetchObject({"_id":ObjectId(tool_iid)})
        tool_vw = ToolView(self.appliTw, self.appliViewFrame, self.mainApp, ToolController(tool_m))
        tool_vw.downloadResultFile()
        
    def deleteToolCallback(self, tool_iid):
        tool_m = Tool.fetchObject({"_id":ObjectId(tool_iid)})
        tool_vw = ToolView(self.appliTw, self.appliViewFrame, self.mainApp, ToolController(tool_m))
        tool_vw.delete()


    def createDefectToolCallback(self, tool_iid):
        tool_m = Tool.fetchObject({"_id":ObjectId(tool_iid)})
        tool_vw = ToolView(self.appliTw, self.appliViewFrame, self.mainApp, ToolController(tool_m))
        tool_vw.createDefectCallback()

    def resetToolCallback(self, tool_iid):
        tool_m = Tool.fetchObject({"_id":ObjectId(tool_iid)})
        tool_vw = ToolView(self.appliTw, self.appliViewFrame, self.mainApp, ToolController(tool_m))
        tool_vw.resetCallback(reload=False)

    def viewScript(self, script):
        ScriptManager.openScriptForUser(script)

    def execScript(self, script):
        data = self.controller.getData()
        data["default_target"] = str(self.controller.getDbId())
        scriptmanager = ScriptManager()
        scriptmanager.executeScript(self.mainApp, script, data, parent=self.mainApp)
    
    @classmethod
    def multiAddInTreeview(self, appliTw, appliViewFrame, mainApp, checkinstances, parent, **kwargs):
        ids = [str(checkinstance.getId()) for checkinstance in checkinstances]
        apiclient = APIClient.getInstance()
        targets_repr = apiclient.getCheckInstanceRepr(ids)
        for checkinstance in checkinstances:
            checkinstance_o = CheckInstanceController(checkinstance)
            checkinstance_o.target_repr = targets_repr.get(str(checkinstance_o.model.target_iid), "Target not found")
            checkinstance_vw = CheckInstanceView(
                appliTw, appliViewFrame, mainApp, checkinstance_o)
            checkinstance_vw.addInTreeview(parent, addChildren=False, detailed=True)
        

    def addInTreeview(self, parentNode=None, **kwargs):
        """Add this view in treeview. Also stores infos in application treeview.
        Args:
            parentNode: if None, will calculate the parent. If setted, forces the node to be inserted inside given parentNode.
        """
        addChildren = kwargs.get("addChildren", True)
        refresh_status = kwargs.get("refresh_status", False)
        self.appliTw.views[str(self.controller.getDbId())] = {"view": self}
        if parentNode is None:
            parentNode = self.getParentNode()
        if kwargs.get("detailed", False):
            text = self.controller.target_repr + " - " +str(self)
        else:
            text = self.controller.target_repr if self.mainApp.settings.is_checklist_view()  else str(self)
        if refresh_status:
            check_infos = self.controller.getCheckInstanceStatus()
        else:
            check_infos = {}
        if kwargs.get("insert_parents", False) and parentNode != "" and parentNode is not None:
            try:
                self.appliTw.insertViewById(parentNode)
            except tk.TclError as e:
                pass
        tags = kwargs.get("tags", None)
        if tags is None:
            tags = self.controller.getTags()
        try:
            self.appliTw.tk.call(self.appliTw, "insert", parentNode, "end", "-id", str(self.controller.getDbId()), "-text", text, "-tags", tags, "-image",self.getIcon(check_infos))
            # ABOVE IS FASTER self.appliTw.insert(parentNode, "end", str(
            #     self.controller.getDbId()), text=text, tags=self.controller.getTags(), image=self.getIcon(check_infos))
        except tk.TclError as e:
            pass
        # if addChildren:
        #     tools = self.controller.getTools()
        #     for tool in tools:
        #         tool_o = ToolController(Tool(tool))
        #         tool_vw = ToolView(
        #             self.appliTw, self.appliViewFrame, self.mainApp, tool_o)
        #         tool_vw.addInTreeview(str(self.controller.getDbId()))
    
        status = check_infos.get("status", "")
        if status == "":
            status = self.controller.model.status
        if status == "":
            status = "todo"
    
        if "hidden" in tags:
            self.hide("tags")
        if  (status != "todo" and self.mainApp.settings.is_show_only_todo()):
            self.hide("filter_todo")
        if self.mainApp.settings.is_show_only_manual() and self.controller.isAuto():
            self.hide("filter_manual")

    def getParentNode(self):
        """
        Return the id of the parent node in treeview.

        Returns:
            return the saved command_node node inside the Appli class.
        """
        if self.mainApp.settings.is_checklist_view():
            res = self._insertParentNode()
            return res
        else:
            parent = self.controller.getTarget()
            if parent is not None and parent != "":
                return parent
        return None

    def _insertParentNode(self):
        datamanager = DataManager.getInstance()
        checkitem = datamanager.get("checkitems", self.controller.model.check_iid)
        if checkitem is not None:
            view = self.mainApp.modelToView("checkitem", checkitem)
            if view:
                view.addInTreeview(tags=[])
        return str(self.controller.model.check_iid)
    
    def _initContextualMenu(self):
        """Initiate contextual menu with variables"""
        self.menuContextuel = utils.craftMenuWithStyle(self.appliViewFrame)

    def popup(self, event):
        """
        Fill the self.widgetMenuOpen and reraise the event in the editing window contextual menu

        Args:
            event: a ttk Treeview event autofilled. Contains information on what treeview node was clicked.
        """
        self.widgetMenuOpen = event.widget
        self.menuContextuel.tk_popup(event.x_root, event.y_root)
        self.menuContextuel.focus_set()
        self.menuContextuel.bind('<FocusOut>', self.popupFocusOut)

    def popupFocusOut(self, _event=None):
        """
        Called when the contextual menu is unfocused
        Args:
            _event: a ttk event autofilled. not used but mandatory.
        """
        self.menuContextuel.unpost()


    def updateReceived(self, obj=None, old_obj=None):
        """Called when a command update is received by notification.
        Update the command treeview item (resulting in icon reloading)
        """
        try:
            self.appliTw.item(str(self.controller.getDbId()), image=self.getIcon())
            self.appliTw.update()
        except tk.TclError as e:
            print(e)
            pass
            #print("WARNING: Update received for a non existing tool "+str(self.controller.getModelRepr()))
        super().updateReceived()

    def key(self):
        """Returns a key for sorting this node
        Returns:
            tuple, key to sort
        """
        return tuple([ord(c) for c in str(self.controller.getModelRepr()).lower()])



"""worker module. Execute code and store results in database, files in the SFTP server.
"""

import errno
import os
import signal
import time
from datetime import datetime, timedelta
from bson.objectid import ObjectId
import shutil
from pollenisatorgui.core.components.apiclient import APIClient
import pollenisatorgui.core.components.utils as utils
from pollenisatorgui.core.components.settings import Settings
from pollenisatorgui.core.models.interval import Interval
from pollenisatorgui.core.models.tool import Tool
import threading
import sys

event_obj = threading.Event()


def sendKill(queue):
    queue.put("kill", block=False)

def executeTool(queue, queueResponse, apiclient, toolId, local=True, allowAnyCommand=False, setTimer=False, infos={}, logger_given=None, worker_name=""):
    """
     remote task
    Execute the tool with the given toolId on the given pentest name.
    Then execute the plugin corresponding.
    Any unhandled exception will result in a task-failed event in the class.

    Args:
        apiclient: the apiclient instance.
        toolId: the mongo Object id corresponding to the tool to execute.
        local: boolean, set the execution in a local context
    Raises:
        Terminated: if the task gets terminated
        OSError: if the output directory cannot be created (not if it already exists)
        Exception: if an exception unhandled occurs during the bash command execution.
        Exception: if a plugin considered a failure.
    """
    import logging
    import sys
    logging.basicConfig(filename='error.log', encoding='utf-8', level=logging.INFO)
    logger = logging.getLogger(__name__)
    handler = logging.StreamHandler(stream=sys.stdout)
    logger.addHandler(handler)

    def handle_exception(exc_type, exc_value, exc_traceback):
        logger.debug("Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback))
        if issubclass(exc_type, KeyboardInterrupt):
            sys.__excepthook__(exc_type, exc_value, exc_traceback)
            sys.exit(1)

        logger.error("Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback))
        sys.__excepthook__(exc_type, exc_value, exc_traceback)
        sys.exit(1)

    sys.excepthook = handle_exception
    #register signals
    signal.signal(signal.SIGTERM, lambda signum, sigframe: sendKill(queue))
    # Connect to given pentest
    logger.debug("executeTool: Execute tool locally:" +str(local)+" setTimer:"+str(setTimer)+" toolId:"+str(toolId))
    APIClient.setInstance(apiclient)
    toolModel = Tool.fetchObject({"_id":ObjectId(toolId)})
    logger.debug("executeTool: get command for toolId:"+str(toolId))
    command_dict = toolModel.getCommand()
    if command_dict is None and toolModel.text != "":
        command_dict = {"plugin":toolModel.plugin_used, "timeout":0}
    msg = ""
    success, data = apiclient.getCommandLine(toolId, toolModel.text)
    comm, fileext = data["comm_with_output"], data["ext"]
    logger.debug("executeTool: got command line for toolId:"+str(toolId))
    if not success:
        print(str(comm))
        logger.debug("Autoscan: Execute tool locally error in getting commandLine : "+str(toolId))
        toolModel.setStatus(["error"])
        sys.exit(1)
    
    outputRelDir = toolModel.getOutputDir(apiclient.getCurrentPentest())
    abs_path = os.path.dirname(os.path.abspath(__file__))
    toolFileName = toolModel.name+"_" + \
            str(time.time()) # ext already added in command
    outputDir = os.path.join(abs_path, "./results", outputRelDir)
    # Create the output directory
    try:
        os.makedirs(outputDir)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(outputDir):
            pass
        else:
            print(str(exc))
            logger.debug("Autoscan: Execute tool locally error in creating output directory : "+str(exc))
            toolModel.setStatus(["error"])
            sys.exit(1)
    outputPath = os.path.join(outputDir, toolFileName)
    comm = comm.replace("|outputDir|", outputPath)
    settings = Settings()
    my_commands = settings.local_settings.get("my_commands", {})
    bin_path = my_commands.get(toolModel.name)
    if bin_path is None:
        if utils.which_expand_alias(command_dict["bin_path"]):
            bin_path = command_dict["bin_path"]
        else:
            toolModel.setStatus(["error"])
            toolModel.notes = str(toolModel.name)+" : no binary path setted"
            logger.debug("Autoscan: Execute tool locally no bin path setted : "+str(toolModel.name))
            sys.exit(1)
    comm = bin_path + " " + comm
    toolModel.updateInfos({"cmdline":comm})
    if "timedout" in toolModel.status:
        timeLimit = None
    # Get tool's wave time limit searching the wave intervals
    elif toolModel.wave == "Custom commands" or (local and not setTimer):
        timeLimit = None
    else:
        timeLimit = getWaveTimeLimit()
    # adjust timeLimit if the command has a lower timeout
    if command_dict is not None and timeLimit is not None:
        timeLimit = min(datetime.now()+timedelta(0, int(command_dict.get("timeout", 0))), timeLimit)
    ##
    try:
        launchableToolId = toolModel.getId()
        name = worker_name
        toolModel.markAsRunning(name, infos)
        logger.debug(f"Mark as running tool_iid {launchableToolId}")
        logger.debug('Autoscan: TASK STARTED:'+toolModel.name)
        logger.debug("Autoscan: Will timeout at "+str(timeLimit))
        print(('TASK STARTED:'+toolModel.name))
        print("Will timeout at "+str(timeLimit))
        # Execute the command with a timeout
        returncode = utils.execute(comm, timeLimit, queue, queueResponse, cwd=outputDir)
        if returncode == -1:
            toolModel.setStatus(["timedout"])
            logger.debug("Autoscan: TOOL timedout at "+str(timeLimit))
            sys.exit(-1)
    except Exception as e:
        print(str(e))
        toolModel.setStatus(["error"])
        logger.debug("Autoscan: TOOL error "+str(e))
        return False, str(e)
    # Execute found plugin if there is one
    outputfile = outputPath+fileext
    plugin = "auto-detect" if command_dict["plugin"] == "" else command_dict["plugin"] 
    msg = apiclient.importToolResult(toolId, plugin, outputfile)
    if msg != "Success":
        #toolModel.markAsNotDone()
        print(str(msg))
        toolModel.setStatus(["error"])
        logger.debug("Autoscan: import tool result error "+str(msg))
        sys.exit(1)
          
    # Delay
    if command_dict is not None:
        print(msg)
    sys.exit(0)
    
def getWaveTimeLimit():
    """
    Return the latest time limit in which this tool fits. The tool should timeout after that limit

    Returns:
        Return the latest time limit in which this tool fits.
    """
    intervals = Interval.fetchObjects({})
    furthestTimeLimit = datetime.now()
    for intervalModel in intervals:
        if utils.fitNowTime(intervalModel.dated, intervalModel.datef):
            endingDate = intervalModel.getEndingDate()
            if endingDate is not None:
                if endingDate > furthestTimeLimit:
                    furthestTimeLimit = endingDate
    return furthestTimeLimit





"""Defines a dialog window for filling a tool settings"""

import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *
from pollenisatorgui.core.forms.formpanel import FormPanel
from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.core.views.viewelement import ViewElement
import pollenisatorgui.core.components.utils as utils

class ChildDialogEditCommandSettings:
    """
    Open a child dialog of a tkinter application to fill settings for a command
    """

    def __init__(self, parent, displayMsg="Choose a database to open", default=None):
        """
        Open a child dialog of a tkinter application to ask a combobox option.

        Args:
            parent: the tkinter parent view to use for this window construction.
            options: A list of string correspondig to options of the combobox
            displayMsg: The message that will explain to the user what he is choosing.
            default: Choose a default selected option (one of the string in options). default is None
        """
        self.app = CTkToplevel(parent)
        self.app.attributes("-type", "dialog")
        self.app.title("Edit command settings")
        self.rvalue = None
        self.parent = parent
        appFrame = CTkFrame(self.app)
        self.app.bind("<Escape>", self.onError)
        self.form = FormPanel()
        self.form.addFormLabel(displayMsg, side=tk.TOP)
        optionsFrame = self.form.addFormPanel(grid=True)
        optionsFrame.addFormLabel("Remote bin path", row=0, column=0)
        optionsFrame.addFormStr("bin", r".+", row=0, column=1)
        optionsFrame.addFormLabel("Plugin", row=1, column=0)
        apiclient = APIClient.getInstance()
        optionsFrame.addFormCombo("plugin", tuple([x["plugin"] for x in apiclient.getPlugins()]), row=1, column=1)
        self.form.addFormButton("Cancel", self.onError, 
                               fg_color=utils.getBackgroundColor(), text_color=utils.getTextColor(),
                               border_width=1, border_color="firebrick1", hover_color="tomato")
        self.form.addFormButton("OK", self.onOk)
        self.form.constructView(appFrame)
        appFrame.pack(ipadx=10, ipady=10)
        try:
            self.app.wait_visibility()
            self.app.transient(parent)
            self.app.focus_force()
            self.app.grab_set()
            self.app.lift()
        except tk.TclError:
            pass

    def onOk(self, _event=None):
        """
        Called when the user clicked the validation button. Set the rvalue attributes to the value selected and close the window.
        """
        # send the data to the parent
        res, msg = self.form.checkForm()
        if not res:
            tk.messagebox.showwarning(
                "Form not validated", msg, parent=self.app)
            return
        form_values = self.form.getValue()
        form_values_as_dicts = ViewElement.list_tuple_to_dict(form_values)
        self.rvalue = (form_values_as_dicts["bin"], form_values_as_dicts["plugin"])
        self.app.destroy()

    def onError(self, _event=None):
        """
        Close the dialog and set rvalue to None
        """
        self.rvalue = None
        self.app.destroy()



"""Widget with a list of checkbox packed and wrapping. A checkbox to check or uncheck all is at the top
#TODO improve looking and constructing"""

import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *
from pollenisatorgui.core.forms.form import Form


class FormChecklist(Form):
    """
    Form field representing a checklist.
    Default setted values: 
        if pack : padx = 10, pady = 5, side = top, fill = "x"
        if grid: row = column = 0
    """
    def __init__(self, name, choicesList, default, values, **kwargs):
        """
        Constructor for a form checklist
        Args:
            name: the checklist name (id).
            choicesList: a list of string forming all the possible choices.
            default: a list of string that should be prechecked if in the choice list.
            values: a list of value to return instead of the strings showing
            kwargs: same keyword args as you would give to CTkFrame
        """
        super().__init__(name)
        self.choicesList = choicesList
        self.default = default
        self.kwargs = kwargs
        self.checkallval = None
        self.checks = []
        self.values = values

    def checkall(self):
        """
        Will check all the checkbox or uncheck same depending on the checkbox state.
        """
        for cb in self.val:
            if self.checkallval.get() == 1:
                cb.set(1)
            else:
                cb.set(0)
        if self.command_check and callable(self.command_check):
            self.command_check()

    def constructView(self, parent):
        """
        Create the checlist view inside the parent view given

        Args:
            FormPanel: parent form panel.
        """
        ################################ THIS is difficult to achieve as the forms positions and width are complicated to get.
        ################################ Temporary fix : hard coded max width.
        self.command_check = self.getKw("command", None)

        frame = CTkFrame(parent.panel)
        self.val = []
        lbl = CTkLabel(frame, text=self.name+" : ")
        lbl.pack(side="top", pady=5, padx=10)
        self.checkallval = tk.IntVar()
        chk = CTkCheckBox(
            frame, text="All", variable=self.checkallval, command=self.checkall)
        chk.pack(side="top", pady=5, padx=10)
        if parent.gridLayout:
            frame.grid(row=self.getKw("row", 0), column=self.getKw("column", 0), **self.kwargs)
        else:
            frame.pack(fill=self.getKw("fill", "x"), side=self.getKw("side", "top"), pady=self.getKw("pady", 5), padx=self.getKw("padx", 10), **self.kwargs)
        container_frame = CTkFrame(frame)
        accx = 0
        for choice in self.choicesList:
            v1 = tk.IntVar()
            if choice in self.default:
                v1.set(1)
            self.val.append(v1)
            chk = CTkCheckBox(container_frame, text=choice,
                                 variable=self.val[-1], command=self.command_check)
            if chk.winfo_reqwidth() + accx >= 400 and choice != self.choicesList[-1]:
                container_frame.pack()
                container_frame = CTkFrame(frame)
                accx = 0
            else:
                accx += chk.winfo_reqwidth()
            self.checks.append(chk)
            chk.pack(side="left", anchor="w", padx=3)
            container_frame.pack()
            if parent.gridLayout:
                frame.grid(row=self.getKw("row", 0), column=self.getKw("column", 0))
            else:
                frame.pack(fill=self.getKw("fill", "x"), side=self.getKw("side", "top"), pady=self.getKw("pady", 5), padx=self.getKw("padx", 10), **self.kwargs)


        if parent.gridLayout:
            frame.grid(row=self.getKw("row", 0), column=self.getKw("column", 0))
        else:
            frame.pack(fill=self.getKw("fill", "x"), side=self.getKw("side", "top"), pady=self.getKw("pady", 5), padx=self.getKw("padx", 10), **self.kwargs)

    def getValue(self):
        """
        Return the form value. Required for a form.

        Returns:
            Return a dictionnary of all checkboxs with texts as keys and 0 or 1 as value. 1 is if the checkbox was ticked.
        """
        ret = {}
        for i, v in enumerate(self.val):
            if self.values is not None:
                ret[self.values[i]] = int(v.get())
            else:
                ret[self.checks[i].cget("text")] = int(v.get())
        return ret

    def setValue(self, newval):
        """Set value of checkboxes defined in given list.
        Args:
            newval: A list with checkbox texts. If a checkbox text matches one in the list, it will checked.
        """
        if isinstance(newval, list):
            newval = [x.lower for x in newval]
        else:
            newval = newval.lower()
        for i, v in enumerate(self.val):
            if self.checks[i].cget("text").lower() in newval:
                v.set(1)
            else:
                v.set(0)

    def checkForm(self):
        """
        Check if this form is correctly filled. A checklist cannot be malformed.

        Returns:
            {
                "correct": True if the form is correctly filled, False otherwise.
                "msg": A message indicating what is not correctly filled.
            }
        """
        return True, ""

    def setFocus(self):
        """Set the focus to the first ttk checkbox of the list."""
        if len(self.checks) > 0:
            self.checks[0].focus_set()


"""Ask the user to select a file or directory and then parse it with the selected parser"""
import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *
import pollenisatorgui.core.components.utils as utils


class ChildDialogAskText:
    """
    Open a child dialog of a tkinter application to ask a text area.
    """

    def __init__(self, parent, info="Input text", default='' , multiline=True, **kwargs):
        """
        Open a child dialog of a tkinter application to ask details about
        existing files parsing.

        Args:
            default_path: a default path to be added
        """
        from pollenisatorgui.core.forms.formpanel import FormPanel
        self.app = CTkToplevel(parent, fg_color=utils.getBackgroundColor())
        self.app.attributes("-type", "dialog")
        fullscreen = False
        if kwargs.get("fullscreen", False):
            fullscreen = True
            monitor = utils.get_screen_where_widget(parent)
            self.app.geometry(f"{monitor.width}x{monitor.height}+{monitor.x}+{monitor.y}")
            del kwargs["fullscreen"]
        self.app.title(info)
        self.rvalue = None
        is_markdown = False
        if "markdown" in kwargs:
            is_markdown = kwargs.get("markdown", True)
            del kwargs["markdown"]
        self.save_on_close = False
        if "save_on_close" in kwargs:
            self.save_on_close = kwargs.get("save_on_close", True)
            del kwargs["save_on_close"]
        self.save_on_close = kwargs.get("save_on_close", True)
        appFrame = CTkFrame(self.app)
        self.form = FormPanel(fill=tk.BOTH, expand=1)
        if not is_markdown:
            self.form.addFormLabel(
                "Input text", text=info, side=tk.TOP)
        if multiline or is_markdown:
            if is_markdown:
                self.formText = self.form.addFormMarkdown(info, "", default,
                                side=tk.TOP, fill=tk.BOTH, expand=1, just_editor=True, allow_maximize=not fullscreen, **kwargs)
            else:
                self.formText = self.form.addFormText(info, "", default,
                                side=tk.TOP, fill=tk.BOTH, expand=1,**kwargs)
        else:
            show = "*" if kwargs.get("secret") else None
            self.formText = self.form.addFormStr(info, "", default, side=tk.TOP, show=show, **kwargs)
        btn = self.form.addFormButton("OK", self.onOk, side=tk.RIGHT)
        self.button = self.form.addFormButton("Cancel", self.onError, side=tk.RIGHT, 
                               fg_color=utils.getBackgroundColor(), text_color=utils.getTextColor(),
                               border_width=1, border_color="firebrick1", hover_color="tomato")

        self.form.constructView(appFrame)
        if not (multiline or is_markdown):
            self.app.bind("<Return>", self.onOk)
        self.app.bind("<Escape>", self.onClose)
        appFrame.pack(ipadx=10, ipady=10, fill=tk.BOTH, expand=1)

        try:
            self.app.wait_visibility()
            self.app.focus_force()
            self.app.grab_set()
            self.app.lift()
        except tk.TclError:
            pass
        self.formText.setFocus()

    def onClose(self, _event=None):
        if self.save_on_close:
            self.onOk()
        else:
            self.onError()

    def onError(self, _event=None):
        self.rvalue = None
        self.app.destroy()

    def onOk(self, _event=None):
        """
        Called when the user clicked the validation button.
        launch parsing with selected parser on selected file/directory.
        Close the window.

        Args:
            _event: not used but mandatory
        """
        res, msg = self.form.checkForm()
        if not res:
            tk.messagebox.showwarning(
                "Form not validated", msg, parent=self.app)
            return
        text = self.formText.getValue()
        self.rvalue = text
        self.app.destroy()



import multiprocessing
import subprocess

import psutil
from pollenisatorgui.core.components.logger_config import logger
import pollenisatorgui.core.components.utils as utils
from pollenisatorgui.core.application.dialogs.ChildDialogQuestion import ChildDialogQuestion
from pollenisatorgui.core.application.dialogs.ChildDialogCombo import ChildDialogCombo
import tempfile
import os
import shutil
import tkinter as tk
from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.scripts.lan.utils import checkPath


def main(apiclient, appli, **kwargs):
    responder_path = utils.which_expand_alias("responder")
    if responder_path is None:
        responder_path = utils.which_expand_alias("Responder.py")
    if responder_path is None:
        responder_path = utils.which_expand_alias("responder.py")
    if responder_path is None:
        return False, "Responder not found, create an alias or install it. (responder, Responder.py, responder.py were tested)"
    res, path = checkPath(["ntlmrelayx.py", "ntlmrelayx"])
    if not res:
        return False, path
    APIClient.setInstance(apiclient)
    smb_signing_list = apiclient.find("computers", {"infos.signing":"False"}, True)
    if smb_signing_list is None or len(smb_signing_list) == 0:
        return False, "No computer with SMB signing disabled found"
    export_dir = utils.getExportDir()
    file_name = os.path.join(export_dir, "relay_list.lst")
    with open(file_name, "w") as f:
        for computer in smb_signing_list:
            ip = computer.get("ip", "")
            if ip != "":
                f.write(ip+"\n")
    relaying_loot_path = os.path.join(export_dir, "loot_relay")
    try:
        os.makedirs(relaying_loot_path)
    except:
        pass
    dialog = ChildDialogQuestion(None, "Setup proxychains", "Do you want to edit proxychains conf to port 1080 ?")
    dialog.app.wait_window(dialog.app)
    cmd = ""
    if dialog.rvalue == "Yes":
        cmd = 'sed -i -E "s/(socks[4-5]\s+127.0.0.1\s+)[0-9]+/\\11080/gm" /etc/proxychains.conf'
        if os.geteuid() != 0:
            cmd = "sudo "+cmd
        appli.launch_in_terminal(None, "sed for proxychains", cmd, use_pollex=False)
    responder_conf = ""
    if utils.which_expand_alias("locate"):
        output = multiprocessing.Queue()
        resp = subprocess.run("locate Responder.conf", capture_output=True, text=True, shell=True)
        stdout = resp.stdout
        if stdout.strip() == "":
            file = tk.filedialog.askopenfilename(title="Locate responder conf file please:", filetypes=[('Config Files', '*.conf')])
            if file:
                responder_conf = file
            else:
                return False, "Responder conf not given"
        else:
            dialog = ChildDialogCombo(None, stdout.split("\n"), displayMsg="Choose your responder config file", width=200)
            dialog.app.wait_window(dialog.app)
            if dialog.rvalue is not None:
                responder_conf = dialog.rvalue.strip()
                if os.geteuid() != 0:
                    cmd = "sudo "+cmd
                cmd = 'sed -i -E "s/(HTTP|SMB) = On/\\1 = Off/gm" '+str(responder_conf)
                appli.launch_in_terminal(None, "sed for responder", cmd, use_pollex=False)
    addrs = psutil.net_if_addrs()
    dialog = ChildDialogCombo(None, addrs.keys(), displayMsg="Choose your ethernet device to listen on")
    dialog.app.wait_window(dialog.app)
    if dialog.rvalue is None:
        return False, "No ethernet device chosen"
    cmd = f"{responder_path} -I {dialog.rvalue} -dvw --lm --disable-ess"
    if os.geteuid() != 0:
        cmd = "sudo "+cmd
    appli.launch_in_terminal(None, "responder", cmd, use_pollex=False)

    cmd = f"{path} -tf {file_name} -smb2support -socks -l {relaying_loot_path}"
    if os.geteuid() != 0:
        cmd = "sudo "+cmd
    appli.launch_in_terminal(kwargs.get("default_target",None), "ntlmrelayx for responder", cmd, use_pollex=False)
    return True, f"Listening ntlmrelay opened, loot directory is here:"+str(relaying_loot_path)+"\n"+ \
            "Don't forget to open Responder with HTTP and SMB disabled\n" + \
                "Proxychains port should be 1080 (default)"



# import os
# import tkinter as tk
# import tkinter.ttk as ttk
# from customtkinter import *
# from multiprocessing import Manager
# import pollenisatorgui.core.components.utils as utils
# from PIL import Image, ImageTk
# from pollenisatorgui.core.views.checkinstanceview import CheckInstanceView
# from pollenisatorgui.core.views.waveview import WaveView
# from pollenisatorgui.core.views.ipview import IpView
# from pollenisatorgui.core.views.portview import PortView
# from pollenisatorgui.core.views.scopeview import ScopeView
# from pollenisatorgui.modules.module import Module

# class Terminal(Module):
#     iconName = "tab_terminal.png"
#     tabName = "   Terminal  "
#     order_priority = Module.LOW_PRIORITY
#     settings = None

#     def __init__(self, parent, settings):
#         super().__init__()
#         self.proc = None
#         self.s = None
#         self.__class__.settings = settings
#         self.img = CTkImage(Image.open(utils.getIcon("help.png")))
#         manager = Manager()
#         self.exiting = manager.Value('i', 0)

#     def initUI(self, parent, nbk, treevw, tkApp):
#         self.treevw = treevw
#         if self.__class__.settings.isTrapCommand():
#             settings_text = "Setting trap command is ON\nEvery command typed here will be executed through pollenisator and will be logged / imported depending on the tools called.\nYou can disable the trap setting in the Settings to change this behaviour."
#         else:
#             settings_text = "Setting trap command is OFF\ntype 'pollex <YOUR COMMAND with --args>' to execute it through pollenisator\n(plugins will autocomplete the output file and import it once done).\n You can enable the trap setting in the Settings to auto-import each commands without prepending pollex."
#         frame = CTkFrame(parent)
#         s = ttk.Style()
#         s.configure('big.TLabel', font=('Helvetica', 12), background="gray97")
#         lbl = CTkLabel(frame,text="", image=self.img)
#         lbl.pack(anchor=tk.CENTER, side=tk.LEFT)
#         lbl = CTkLabel(frame, text=settings_text)
#         lbl.pack(anchor=tk.CENTER, side=tk.RIGHT)
#         frame.place(relx=.5, rely=.5, anchor="c")
#         return

#     def open(self):
#         self.__class__.openTerminal()
#         return True

#     @classmethod
#     def openTerminal(cls, default_target=None):
        
            
#         if cls.settings.isTrapCommand():
#             comm = "bash --rcfile "+os.path.join(utils.getMainDir(), "setupTerminalForPentest.sh")
#         else:
#             comm = "bash"
        
#         res = utils.executeInExternalTerm(comm, with_bash=False,  default_target=default_target)
        

#     def onClosing(self, _signum=None, _frame=None):
#         #self.exiting.value = 1
#         if self.s:
#             self.s.close()

#     def _initContextualsMenus(self, parentFrame):
#         """
#         Create a contextual menu
#         """
#         self.contextualMenu = utils.craftMenuWithStyle(parentFrame)
#         self.contextualMenu.add_command(
#             label="Attack from terminal", command=self.attackFromTerminal)
#         return self.contextualMenu

#     def attackFromTerminal(self, _event=None):
#         for selected in self.treevw.selection():
#             view_o = self.treevw.getViewFromId(selected)
#             if view_o is not None:
#                 if isinstance(view_o, CheckInstanceView):
#                     Terminal.openTerminal(view_o.controller.getDbId())
#                 else:
#                     lvl = "network" if isinstance(view_o, ScopeView) else None
#                     lvl = "wave" if isinstance(view_o, WaveView) else lvl
#                     lvl = "ip" if isinstance(view_o, IpView) else lvl
#                     lvl = "port" if isinstance(view_o, PortView) else lvl
                    
                    
#                     if lvl is not None:
#                         inst = view_o.controller.getData()
#                         Terminal.openTerminal(lvl+"|"+inst.get("wave", "Imported")+"|"+inst.get(
#                             "scope", "")+"|"+inst.get("ip", "")+"|"+inst.get("port", "")+"|"+inst.get("proto", ""))
#                     else:
#                         tk.messagebox.showerror(
#                             "ERROR : Wrong selection", "You have to select a object that may have tools")



import tkinter as tk
import tkinter.ttk as ttk
import uuid
from customtkinter import *
import pollenisatorgui.core.components.utils as utils
from pollenisatorgui.core.components.settings import Settings
from pollenisatorgui.core.application.paginable import Paginable
import pyperclip
from collections import OrderedDict


class ScrollableTreeview(Paginable):
    def __init__(self, root, columns, **kwargs):
        if not kwargs.get("paginate", True):
            maxPerPage = -1
        else:
            maxPerPage = kwargs.get("maxPerPage", 10)
        super().__init__(root, self.insert_items, self.empty_treeview, self.callback_get_value, lambda: 0, height=0, maxPerPage=maxPerPage)
        self.root = root
        self.columns = columns
        self._detached = set()
        self.autoresize = kwargs.get("autoresize", True)
        self.sort_keys = kwargs.get("sort_keys", None)
        self.content_view = self.getContentView()
        self.treevw = ttk.Treeview(self.content_view, style=kwargs.get("style",None), height=max(kwargs.get("height", 10), maxPerPage))
        if len(columns) > 1:
            self.treevw['columns'] = columns[1:]
        settings = Settings()
        self.treevw.tag_configure("odd", background=utils.getBackgroundSecondColor())
        lbl = CTkLabel(self)
        self.f = tk.font.Font(lbl, "Sans", bold=True, size=10)
        width = kwargs.get("width", None)
        if width is not None:
            w_per_column = width // len(self.columns)
        self.columnsLen = [self.f.measure(column) for column in self.columns]
        listOfLambdas = [self.column_clicked("#"+str(i), False) for i in range(len(self.columns))]
        for h_i, header in enumerate(self.columns):
            self.treevw.heading("#"+str(h_i), text=header, anchor="w", command=listOfLambdas[h_i])
            if width is not None:
                self.treevw.column("#"+str(h_i), anchor='w',
                                   stretch=tk.YES, minwidth=w_per_column, width=w_per_column)
            else:
                self.treevw.column("#"+str(h_i), anchor='w',
                               stretch=tk.YES, minwidth=self.columnsLen[h_i])
        self.treevw.grid(row=0, column=0, sticky=tk.NSEW)
        for bindName, callback in kwargs.get("binds", {}).items():
            self.treevw.bind(bindName, callback)
        scbVSel = CTkScrollbar(self.content_view,
                                orientation=tk.VERTICAL,
                                command=self.treevw.yview)
        scbHSel = CTkScrollbar(
            self.content_view, orientation=tk.HORIZONTAL, command=self.treevw.xview)
        self.treevw.configure(yscrollcommand=scbVSel.set)
        self.treevw.configure(xscrollcommand=scbHSel.set)
        scbVSel.grid(row=0, column=1, sticky=tk.NS)
        scbHSel.grid(row=1, column=0, sticky=tk.EW)
        self.setPaginationPanel()
        
       
        self.treevw.bind("<Control-c>", self.copy)
        self.treevw.bind('<Control-a>', self.selectAll)
        self.treevw.bind("<Escape>", self.unselect)
        self._initContextualMenu(self.treevw)

    def empty_treeview(self):
        for item in self.treevw.get_children():
            self.treevw.delete(item)

    def unselect(self, event=None):
        for item in self.treevw.selection():
            self.treevw.selection_remove(item)
            
    def callback_get_value(self, item, column):
        if column == 0:
            return item["text"]
        else:
            return item["values"][column-1]

    def selectAll(self, event=None):
        self.treevw.selection_set(self.treevw.get_children())

    def bind(self, event_name, func):
        self.treevw.bind(event_name, func)

    def insert(self, parent, index, iid, text="",values=(), tags=(), image=None, auto_update_pagination=True):
        res = None
        if iid is None:
            iid = str(uuid.uuid4())
        if str(iid) not in self.infos:
            res = self.addPaginatedInfo({"parent":parent,"iid":iid, "index":index, "text":text,"values":values,"tags":tags, "image":image}, auto_update_pagination=auto_update_pagination)
        return res
    
    def insert_items(self, items):
        res = None
        for t in items:
            res = self._insert(t["parent"], t["index"], t["iid"], t["text"], t["values"], t["tags"], t["image"])
        return res
    
    def _insert(self, parent, index, iid, text="",values=(), tags=(), image=None):
        kwargs = {}
        if image is not None:
            kwargs["image"] =image
        try:
            res = self.treevw.insert(parent, index, iid, text=text, values=values, tags=tags, **kwargs)
        except tk.TclError as e:
            return None
            
        if self.autoresize:
            self.columnsLen[0] = max(self.columnsLen[0], self.f.measure(text))
            self.treevw.column("#0", anchor='w',
                                stretch=tk.YES, minwidth=self.columnsLen[0], width=self.columnsLen[0])
            for i, val in enumerate(values):
                self.columnsLen[i+1] = min(1000, max(self.columnsLen[i+1], self.f.measure(str(val))))
                self.treevw.column("#"+str(i+1), anchor='w',
                                stretch=tk.YES, minwidth=self.columnsLen[i+1], width=self.columnsLen[i+1])
            # self.treevw.grid_forget()
            # self.treevw.grid(row=0, column=0, sticky=tk.NSEW) # TODO : DO BETTER FOR redraw...
        self.resetOddTags()
        return res

    def item(self, iid, **kwargs):
        try:
            self.treevw.item(iid, **kwargs)
        except tk.TclError as e:
            pass
        try:
            self.infos[str(iid)].update(kwargs)
        except ValueError as e:
            raise tk.TclError(e)
        return self.infos[str(iid)]
        
        
    def _initContextualMenu(self, parent):
        """Initialize the contextual menu for paperclip.
        Args:
            parent: the tkinter parent widget for the contextual menu
        """
        self.contextualMenu = utils.craftMenuWithStyle(parent)
        parent.bind("<Button-3>", self.popup)
        self.contextualMenu.add_command(label="Copy", command=self.copy)
        self.contextualMenu.add_command(label="Close", command=self.close)

        
    def addContextMenuCommand(self, label, command, replace=False):
        found = False
        for i in range(self.contextualMenu.index('end')+1):
            labelStr = str(self.contextualMenu.entrycget(i,'label') )
            if labelStr == label and not replace:
                found = True
                break
        if not found:
            self.contextualMenu.add_command(label=label, command=command)

    def close(self):
        """Option of the contextual menu : Close the contextual menu by doing nothing
        """
        pass

    def copy(self, _event=None):
        """Option of the contextual menu : Copy entry text to clipboard
        """
        selected = self.treevw.selection()
        texts = []
        for item in selected:
            it = self.item(item)
            texts.append(it.get("text", "") + " " +
                         " ".join(map(str,it.get("values", []))))

        pyperclip.copy("\n".join(texts))

    def popup(self, event):
        """
        Fill the self.widgetMenuOpen and reraise the event in the editing window contextual menu

        Args:
            event: a ttk Treeview event autofilled.
            Contains information on what treeview node was clicked.
        """
        self.widgetMenuOpen = event.widget
        self.contextualMenu.tk_popup(event.x_root, event.y_root)
        self.contextualMenu.focus_set()
        self.contextualMenu.bind('<FocusOut>', self.popupFocusOut)

    def popupFocusOut(self, _event=None):
        """Callback for focus out event. Destroy contextual menu
        Args:
            _event: not used but mandatory
        """
        self.contextualMenu.unpost()

    

    

    def detach(self, item_id):
        try:
            self.treevw.detach(item_id)
            self._detached.add(item_id)
        except tk.TclError:
            pass

    def reattach(self, item_id, parent, index):
        try:
            self.treevw.reattach(item_id, parent, index)
            self._detached.discard(item_id)
        except tk.TclError:
            pass
    
    @classmethod
    def date_compare(cls, start, end, toCompare):
        dated = utils.stringToDate(start)
        datef = utils.stringToDate(end)
        toCompare = utils.stringToDate(toCompare)
        if dated is None or datef is None:
            return True
        return dated <= toCompare <= datef

    def column_clicked(self, col, reverse):
        return lambda : self.sort_column(self.treevw, col, reverse)

    def sort_column(self, tv, col, reverse):
        sort_key = None
        if self.sort_keys:
            sort_key = self.sort_keys[int(col[1:])]
        if sort_key is None:
            sort_key = str
        if col == "#0":
            sorted_values = sorted(self.infos.values(), key=lambda info: sort_key(info["text"]), reverse=reverse)
        else:
            sorted_values = sorted(self.infos.values(), key=lambda info: sort_key(str(info["values"][int(col[1:])-1])), reverse=reverse)
        new_infos = OrderedDict()
        for info in sorted_values:
            new_infos[info["iid"]] = info
        self.infos = new_infos
        tv.heading(col, command=self.column_clicked(col, not reverse))
        self.goToPage("first", force=True)

    def reset(self):
        """Reset the treeview values (delete all lines)"""
        for item in self.treevw.get_children():
            self.treevw.delete(item)
        self.infos = OrderedDict()
        self._detached = set()
        self.resetPagination()


    def resetOddTags(self):
        for i, child in enumerate(self.treevw.get_children()):
            odd_tag = ("odd") if i%2 != 0 else ()
            current_tags = self.item(child)["tags"]
            current_tags = [current_tags] if isinstance(current_tags, str) else list(current_tags)
            if "odd" in current_tags:
                current_tags.remove("odd")
            self.item(child, tags=[odd_tag]+current_tags)

    def delete(self, _event=None):
        """Callback for <Del> event
        Remove the selected item in the treeview
        Args:
            _event: not used but mandatory"""
        for selected in self.treevw.selection():
            try:
                item = self.item(selected)
                if item["text"].strip() != "":
                    self.treevw.delete(selected)
                    try:
                        del self.infos[selected]
                    except ValueError as e:
                        pass
            except tk.TclError:
                pass
        self.resetOddTags()

    def selection(self):
        return self.treevw.selection()
    
    def get_children(self, all=False):
        if all:
            return list(self.infos.keys())
        return self.treevw.get_children()
    
    def identify(self, *args, **kwargs):
        return self.treevw.identify(*args, **kwargs)
    
    def identify_column(self, *args, **kwargs):
        return self.treevw.identify_column(*args, **kwargs)

    def parent(self, item):
        return self.treevw.parent(item)

    

        


from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.core.components.datamanager import DataManager
from pollenisatorgui.core.models.element import Element
class User(Element):
    coll_name = "users"
    
    def __init__(self, valuesFromDb=None):
        if valuesFromDb is None:
            valuesFromDb = {}
        super().__init__(valuesFromDb.get("_id", None), valuesFromDb.get("parent", None),  valuesFromDb.get("infos", {}))
        self.initialize(valuesFromDb.get("_id"), valuesFromDb.get("domain"), valuesFromDb.get("username"), valuesFromDb.get("password"),
            valuesFromDb.get("groups"), valuesFromDb.get("description"), valuesFromDb.get("infos", {}))

    def initialize(self, _id, domain=None, username=None, password=None,groups=None, description=None, infos=None):
        """User
        :param _id: iid of the object
        :type _id: str
        :param username: The username of this User.
        :type username: str
        :param password: The password of this User.
        :type password: str
        :param domain: The domain of this User.
        :type domain: str
        :param groups: The groups of this User.
        :type groups: List[str]
        :param description: The description of this User.
        :type description: str
        """
      
        self._id = _id
        self.username = username if username is not None else  ""
        self.password = password if password is not None else  ""
        self.domain = domain if domain is not None else  ""
        self.groups = groups
        self.description = description
        self.infos =  infos if infos is not None else {}
        return self
  
    def __str__(self):
        """
        Get a string representation of a defect.

        Returns:
            Returns the defect +title.
        """
        return self.domain+"\\"+self.username 

    def getData(self):
        return {"_id": self._id, "username":self.username, "password": self.password, "domain":self.domain,
         "groups": self.groups, "description":self.description, "infos":self.infos}

    
    @classmethod
    def fetchObjects(cls, pipeline):
        """Fetch many commands from database and return a Cursor to iterate over model objects
        Args:
            pipeline: a Mongo search pipeline (dict)
        Returns:
            Returns a cursor to iterate on model objects
        """
        apiclient = APIClient.getInstance()
        pipeline["type"] = "user"
        ds = apiclient.find(cls.coll_name, pipeline, True)
        if ds is None:
            return None
        for d in ds:
            # disabling this error as it is an abstract function
            yield User(d)  #  pylint: disable=no-value-for-parameter
    
    @classmethod
    def fetchObject(cls, pipeline):
        """Fetch many commands from database and return a Cursor to iterate over model objects
        Args:
            pipeline: a Mongo search pipeline (dict)
        Returns:
            Returns a cursor to iterate on model objects
        """
        apiclient = APIClient.getInstance()
        pipeline["type"] = "user"
        d = apiclient.find(cls.coll_name, pipeline, False)
        if d is None:
            return None
        return User(d)
    
    @classmethod
    def fetchPentestObjects(cls):
        return [x for x in User.fetchObjects({})]
    
    def _getParentId(self):
        """
        Return the mongo ObjectId _id of the first parent of this object. For a port it is the ip.

        Returns:
            Returns the parent ip's ObjectId _id".
        """
        return self.domain


"""ActiveDirectory module"""
import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *
import os
import re

from pollenisatorgui.modules.ActiveDirectory.controllers.sharecontroller import ShareController

from pollenisatorgui.modules.ActiveDirectory.controllers.computercontroller import ComputerController
from pollenisatorgui.modules.ActiveDirectory.views.shareview import ShareView
from pollenisatorgui.core.application.scrollableframexplateform import ScrollableFrameXPlateform
from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.core.application.dialogs.ChildDialogProgress import ChildDialogProgress
from pollenisatorgui.core.application.dialogs.ChildDialogQuestion import ChildDialogQuestion
from pollenisatorgui.core.application.dialogs.ChildDialogAskText import ChildDialogAskText
from pollenisatorgui.core.application.dialogs.ChildDialogAskFile import ChildDialogAskFile
from pollenisatorgui.core.application.scrollabletreeview import ScrollableTreeview
from pollenisatorgui.core.components.datamanager import DataManager
from pollenisatorgui.core.components.tag import TagInfos
from pollenisatorgui.core.models.port import Port
from pollenisatorgui.core.forms.formpanel import FormPanel
from pollenisatorgui.modules.ActiveDirectory.controllers.computercontroller import ComputerController
from pollenisatorgui.modules.ActiveDirectory.controllers.usercontroller import UserController
from pollenisatorgui.modules.ActiveDirectory.views.computerview import ComputerView
from pollenisatorgui.modules.ActiveDirectory.views.userview import UserView
from pollenisatorgui.modules.module import Module
from pollenisatorgui.modules.ActiveDirectory.models.users import User # load it in registry
from pollenisatorgui.modules.ActiveDirectory.models.computers import Computer # load it in registry
from pollenisatorgui.modules.ActiveDirectory.models.shares import Share # load it in registry
import tempfile
from bson import ObjectId
import pollenisatorgui.core.components.utils as utils
from pollenisatorgui.core.components.settings import Settings
from PIL import Image
from pollenisatorgui.core.application.pollenisatorentry import PopoEntry


class ActiveDirectory(Module):
    """
    Shows information about ongoing pentest. 
    """
    iconName = "tab_AD.png"
    tabName = "Active Directory"
    coll_name = "ActiveDirectory"
    classes = ["users", "computers", "shares"]
    order_priority = Module.HIGH_PRIORITY
    settings = Settings()
    pentest_types = ["lan"]

    def __init__(self, parent, settings, tkApp):
        """
        Constructor
        """
        super().__init__()
        self.parent = None
        self.users = {}
        self.treevw = None
        self.tkApp = tkApp
        self.computers = {}
        self.shares = {}
        self.inited = False
        
    

    @classmethod
    def getSettings(cls):
        return cls.settings.local_settings.get(ActiveDirectory.coll_name, {})

    @classmethod
    def saveSettings(cls, newSettings):
        cls.settings.local_settings[ActiveDirectory.coll_name] = newSettings
        cls.settings.saveLocalSettings()

    def open(self, view, nbk, treevw):
        apiclient = APIClient.getInstance()
        self.treevw = treevw
        if self.inited is False:
            self.initUI(view)
        if apiclient.getCurrentPentest() is not None:
            self.refreshUI()
        return True
    
    def onTreeviewLoad(self, checklistview, lazyload):
        """
        Called when the treeview is loaded
        """
        datamanager = DataManager.getInstance()
        if not checklistview and not lazyload:
            computers = list(datamanager.get("computers", '*'))
            for computer in computers:
                computer_o = ComputerController(computer)
                computer_vw = ComputerView(self.tkApp.treevw, self.tkApp.viewframe, self.tkApp, computer_o)
                computer_vw.addInTreeview(None, addChildren=False)
            shares = list(datamanager.get("shares", '*'))
            for share in shares:
                share_o = ShareController(share)
                share_vw = ShareView(self.tkApp.treevw, self.tkApp.viewframe, self.tkApp, share_o)
                share_vw.addInTreeview(None, addChildren=False)
        if not checklistview:
            users = list(datamanager.get("users", '*'))
            for user in users:
                user_o = UserController(user)
                user_vw = UserView(self.tkApp.treevw, self.tkApp.viewframe, self.tkApp, user_o)
                user_vw.addInTreeview(None, addChildren=False)
            

    def refreshUI(self):
        """
        Reload data and display them
        """
        
        self.loadData()
        self.displayData()
        self.reloadSettings()

    def loadData(self):
        """
        Fetch data from database
        """
        apiclient = APIClient.getInstance()
        dialog = ChildDialogProgress(self.parent, "Loading infos ",
                                     "Refreshing infos. Please wait for a few seconds.", 200, "determinate")
        dialog.show(5)
        dialog.update(0)
        self.users = apiclient.find("users", {"type": "user"}, True)
        dialog.update(1)
        if self.users is None:
            self.users = []
        self.computers = apiclient.find("computers", {"type": "computer"}, True)
        dialog.update(2)
        if self.computers is None:
            self.computers = []
        self.shares = apiclient.find("shares", {"type": "share"}, True)
        dialog.update(3)
        if self.shares is None:
            self.shares = []
        dialog.update(4)
        self.tags = {}
        find_tags = apiclient.find("tags", {"item_type": {"$in":["computers","users","shares"]}}, True)
        for tag in find_tags:
            self.tags[str(tag.get("item_id"))] = tag
        dialog.destroy()
            
    def displayData(self):
        """
        Display loaded data in treeviews
        """
        dialog = ChildDialogProgress(self.parent, "Displaying infos ",
                                     "Refreshing infos. Please wait for a few seconds.", 200, "determinate")
        dialog.show(4)

        self.tvUsers.reset()
        self.tvComputers.reset()
        self.tvShares.reset()
        dialog.update(1)
        for i,user in enumerate(self.users):
            self.insertUser(user,  auto_update_pagination=False)
        self.tvUsers.setPaginationPanel()
        dialog.update(2)
        for i,computer in enumerate(self.computers):
            self.insertComputer(computer, auto_update_pagination=False)
        self.tvComputers.setPaginationPanel()
        
        dialog.update(3)
        for i,share in enumerate(self.shares):
            self.insertShare(share, auto_update_pagination=False)
        self.tvShares.setPaginationPanel()
        dialog.update(4)
        dialog.destroy()

    def statusbarClicked(self, tagname):
        self.searchBar.delete(0, tk.END)
        self.searchBar.insert(tk.END, tagname)
        self.search()

    def modelToView(self, data_type, model):
        if data_type.lower() == "users":
            return UserView(self.tkApp.treevw, self.tkApp.viewframe, self.tkApp, UserController(model))
        elif data_type.lower() == "computers":
            return ComputerView(self.tkApp.treevw, self.tkApp.viewframe, self.tkApp, ComputerController(model))
        elif data_type.lower() == "shares":
            return ShareView(self.tkApp.treevw, self.tkApp.viewframe, self.tkApp, ShareController(model))
        return None

    def mapLambda(self, c, treeview=None, model=None):
        if c[0].lower() in ["computer"]:
            return lambda : self.computerCommand(c[2], treeview=treeview, model=model)
        elif c[0].lower() == "share":
            return lambda : self.shareCommand(c[2], treeview=treeview, model=model)

    def reloadSettings(self):
        s = self.getSettings()
        listOfLambdas = [self.mapLambda(c) for c in s.get("commands", [])]
        i = 0
        for command_options in s.get("commands", []):
            if command_options[0].lower() == "computer":
                self.tvComputers.addContextMenuCommand(command_options[1], listOfLambdas[i], replace=True)
            elif command_options[0].lower() == "share":
                self.tvShares.addContextMenuCommand(command_options[1], listOfLambdas[i], replace=True)
            i+=1
        
    def getAdditionalContextualCommands(self, type, treeview, view):
        s = self.getSettings()
        listOfLambdas = [self.mapLambda(c, treeview, view.controller.model) for c in s.get("commands", [])]
        i = 0
        ret = {}
        for command_options in s.get("commands", []):
            if command_options[0].lower() == type.lower():
                ret[command_options[1]] = listOfLambdas[i]
            i+=1
        return ret

    def initUI(self, parent):
        """
        Initialize Dashboard widgets
        Args:
            parent: its parent widget
        """
        self.inited = True
        self.parent = parent
        settings_btn = CTkButton(parent, text="Configure this module", command=self.openConfig)
        settings_btn.pack(side="bottom", pady=5)
        self.moduleFrame = ScrollableFrameXPlateform(parent)
        frameSearchBar = CTkFrame(self.moduleFrame)
        self.searchBar = PopoEntry(frameSearchBar, placeholder_text="Search")
        self.searchBar.pack(side="left", fill=tk.X, expand=True)
        self.searchBar.bind("<KeyRelease>", self.search)
        frameSearchBar.pack(side="top", fill=tk.X, expand=True)
        self.frameTreeviews = CTkFrame(self.moduleFrame)
        frameUsers = CTkFrame(self.frameTreeviews)
        self.tvUsers = ScrollableTreeview(
            frameUsers, ("Username", "Password", "Domain", "N° groups","Desc", "hashNT", "Tags"), binds={"<Delete>":self.deleteUser, "<Double-Button-1>":self.userDoubleClick})
        self.tvUsers.pack(fill=tk.BOTH)
        addUserButton = CTkButton(frameUsers, text="Add user manually", command=self.addUsersDialog)
        addUserButton.pack(side="bottom")
        frameComputers = CTkFrame(self.frameTreeviews)
        self.tvComputers = ScrollableTreeview(
            frameComputers, ("IP", "Name", "Domain", "DC", "Admin count", "User count", "OS", "Signing", "SMBv1", "Tags"), width=parent.winfo_width()-100, binds={"<Double-Button-1>":self.computerDoubleClick})
        self.tvComputers.pack(fill=tk.BOTH)
        frameShares = CTkFrame(self.frameTreeviews)
        self.tvShares = ScrollableTreeview(
            frameShares, ("IP", "Share", "Flagged", "Size", "Tags"), paginate=False)
        self.tvShares.pack(fill=tk.BOTH)
        frameUsers.grid(row=0, column=0)
        frameComputers.grid(row=1, column=0)
        frameShares.grid(row=2, column=0)
        self.frameTreeviews.columnconfigure(0, weight=1)
        self.frameTreeviews.pack(padx=10, pady=10, fill=tk.BOTH, expand=True)
        self.moduleFrame.pack(padx=10, pady=10, fill=tk.BOTH, expand=True)

    def search(self, event=None):
        """
        Search in treeviews
        """
        if event is not None:
            if event.keysym != "BackSpace" and event.keysym != "Delete" and (len(event.keysym) > 1 or not event.keysym.isalnum()):
                return
        search = self.searchBar.get()
        self.tvUsers.filter(search, search, search, search, search, search, search, check_all=False, check_case=False)
        self.tvComputers.filter(search, search, search, search, search, search, search, search, search, search, check_all=False, check_case=False)
        self.tvShares.filter(search, search, search, search, search, check_all=False, check_case=False)

    def computerDoubleClick(self, event=None):
        selection = self.tvComputers.selection()
        if selection:
            self.openComputerDialog(selection[0])

    def openComputerDialog(self, computer_iid):
        self.getComputerDialog(computer_iid)

    def getComputerDialog(self, computer_iid):
        apiclient = APIClient.getInstance()
        computer_d = apiclient.find("computers",
            {"_id":ObjectId(computer_iid)}, False)
        if computer_d is None:
            return
        cw = ComputerView(self.treevw, None, self.tkApp, ComputerController(Computer(computer_d)))
        cw.openInDialog(False)

    def insertUser(self, user, auto_update_pagination=True):
        try:
            domain = user.get("domain", "")
            username = user.get("username", "")
            password = user.get("password", "")
            groups = user.get("groups", [])
            datamanager = DataManager.getInstance()
            if groups is None:
                groups = []
            user_tags = self.tags.get(str(user["_id"]), {})
            tags_info = user_tags.get("tags", [])
            tags_names = [tag.get("name", "") for tag in tags_info if tag.get("name", "") != ""]
            self.tvUsers.insert('', 'end', user["_id"], text=username, 
                                values=(password, domain, str(len(groups)), user.get("description", ""), user.get("infos", {}).get("hashNT", ""),", ".join(tags_names)), auto_update_pagination=auto_update_pagination)
        except tk.TclError as e:
            pass

    def userDoubleClick(self, event=None):
        selection = self.tvUsers.selection()
        if selection:
            self.openUserDialog(selection[0])

    def openUserDialog(self, user_iid):
        self.getUserDialog(user_iid)

    def getUserDialog(self, user_iid):
        apiclient = APIClient.getInstance()
        user_d = apiclient.find("users",
            {"_id":ObjectId(user_iid)}, False)
        if user_d is None:
            return
        user_vw = UserView(self.treevw, None, self.tkApp, UserController(User(user_d)))
        user_vw.openInDialog(False)

    def deleteUser(self, event=None):
        apiclient = APIClient.getInstance() 
        selection = self.tvUsers.selection()
        users_iid = [str(x["_id"]) for x in self.users]
        for select in selection:
            try:
                item = self.tvUsers.item(select)
            except tk.TclError:
                pass
            apiclient.delete( ActiveDirectory.coll_name+"/users", select)
            try:
                index = users_iid.index(str(select))
                del users_iid[index]
                del self.users[index]
            except ValueError:
                pass

            self.tvUsers.delete(select)
    
    def insertComputer(self, computer, auto_update_pagination=True):
        infos = computer.get("infos",{})
        computer_tags = self.tags.get(str(computer["_id"]), {})
        tags_info = computer_tags.get("tags", [])
        tags_names = [tag.get("name", "") for tag in tags_info if tag.get("name", "") != ""]
        newValues = (computer.get("name",""), computer.get("domain", ""), infos.get("is_dc", False), len(computer.get("admins", [])), len(computer.get("users", [])), infos.get("os", ""), \
                        infos.get("signing", ""), infos.get("smbv1", ""), ", ".join(tags_names))
        
        try:
            self.tvComputers.insert(
                '', 'end', computer["_id"], text=computer.get("ip", ""),
                values=newValues, auto_update_pagination=auto_update_pagination)
        except tk.TclError:
            self.tvComputers.item(computer["_id"], values=newValues) 

    def insertShare(self, share, auto_update_pagination=True):
        share_tags = self.tags.get(str(share["_id"]), {})
        tags_info = share_tags.get("tags", [])
        tags_names = [tag.get("name", "") for tag in tags_info if tag.get("name", "") != ""]
        try:
            parentiid = self.tvShares.insert(
                        '', 'end', share["_id"], text=share.get("ip", ""), values=(share.get("share", ""), ", ".join(tags_names),"",""), auto_update_pagination=auto_update_pagination)
        except tk.TclError:
            parentiid = str(share["_id"])
        for file_infos in share.get("files",[]):
            if not file_infos["flagged"]:
                continue
            toAdd = (file_infos["path"], str(file_infos["flagged"]), str(file_infos["size"]), "")
            try:
                self.tvShares.insert(
                    parentiid, 'end', None, text="", values=tuple(toAdd), auto_update_pagination=auto_update_pagination)
            except tk.TclError:
                pass

    def addUserInDb(self, domain, username, password):
        apiclient = APIClient.getInstance()
        user = {"type": "user", "username": username, "domain": domain, "password": password}
        res = apiclient.insert( ActiveDirectory.coll_name+"/users", {"username": username, "domain": domain, "password": password})
        
     
    def update_received(self, dataManager, notif, obj, old_obj):
        if not self.inited:
            return
        if notif["collection"] not in ActiveDirectory.classes:
            return
        apiclient = APIClient.getInstance()
        res = apiclient.find(notif["collection"], {"_id": ObjectId(notif["iid"])}, False)
        iid = notif["iid"]
        if notif["action"] == "insert":
            if res is None:
                return
            if notif["collection"] == "computers":
                self.insertComputer(res)
            elif notif["collection"] == "users":
                self.insertUser(res)
            elif notif["collection"] == "shares":
                self.insertShare(res)
        elif notif["action"] == "update":
            if res is None:
                return
            if notif["collection"] == "computers":
                self.insertComputer(res)
            if notif["collection"] == "shares":
                self.insertShare(res)
        elif notif["action"] == "delete":
            if res is None:
                return
            try:
                if notif["collection"] == "computers":
                    self.tvComputers.delete(str(iid))
                elif notif["collection"] == "users":
                    self.tvUsers.delete(str(iid))
                elif notif["collection"] == "shares":
                    self.tvShares.delete(str(iid))
            except tk.TclError:
                pass

    def openConfig(self, event=None):
        dialog = ChildDialogConfigureADModule(self.parent)
        self.parent.wait_window(dialog.app)
        if dialog.rvalue is not None and isinstance(dialog.rvalue, list):
            settings = {"commands":[]}
            for values in dialog.rvalue:
                settings["commands"].append(values)
            self.saveSettings(settings)
        self.reloadSettings()
            
    def addUsersDialog(self, event=None):
        dialog = ChildDialogAddUsers(self.parent)
        self.parent.wait_window(dialog.app)
        if dialog.rvalue is not None and isinstance(dialog.rvalue, list):
            lines = dialog.rvalue
            for line in lines:
                if line.strip() != "":
                    parts = line.split("\\")
                    domain = parts[0]
                    remaining = "\\".join(parts[1:])
                    parts = remaining.split(":")
                    username = parts[0]
                    password = ":".join(parts[1:])
                    self.addUserInDb(domain, username, password)
    
    def exportAllUsersAsFile(self, domain="", delim="\n"):
        fp = tempfile.mkdtemp()
        filepath = os.path.join(fp, "users.txt")
        with open(filepath, mode="w") as f:
            apiclient = APIClient.getInstance()
            search = {"type":"user"}
            if domain != "":
                search["domain"] = domain 
            res = apiclient.find("users", search)
            for selected_user in res:
                username = selected_user["username"]
                f.write(username+delim)
        return filepath

    def exportAllComputersAsFile(self, delim="\n"):
        fp = tempfile.mkdtemp()
        filepath = os.path.join(fp, "computers.txt")
        with open(filepath, mode="w") as f:
            apiclient = APIClient.getInstance()
            res = apiclient.find("computers", {"type":"computers"})
            for selected_computer in self.tvComputers.get_children():
                computer = self.tvComputers.item(selected_computer)["text"]
                f.write(computer+delim)
        return filepath


    def shareCommand(self, command_option, treeview=None, model=None):
        if treeview is None:
            treeview = self.tvShares
        selected = treeview.selection()[0]
        if model is None:
            parent_iid = treeview.parent(selected)
            if not parent_iid: # file in share
                return
            ip = treeview.item(parent_iid)["text"]
            apiclient = APIClient.getInstance()
            share_m = apiclient.find("shares", {"_id": ObjectId(parent_iid)}, False)
            path = item_values[0]
            share_name = item_values[0]
        else:
            ip = model.ip
            share_m = model.getData()
            path = treeview.item(selected)["text"]
            share_name = model.share
            

        item_values = treeview.item(selected)["values"]
        if share_m is None:
            return
        
        files = share_m.get("files", [])
        try:
            path_index = [x["path"] for x in files].index(path)
        except:
            tk.messagebox.showerror("path not found","Path "+str(path)+" was not found in share")
            return
        
        file_infos = files[path_index]
        users = file_infos.get("users", [])
        if not users:
            tk.messagebox.showerror("No users known","This share has no no known user ")
            return
        u = users[0] # take first one
        domain = u[0] if u[0] is not None else ""
        user = u[1] if u[1] is not None else ""
        
        apiclient = APIClient.getInstance()
        apiclient.getCurrentPentest()
        user_o = apiclient.find("users", 
                {"type":"user", "domain":domain, "username":user}, False)
        if user_o is None:
            tk.messagebox.showerror("user not found","User "+str(domain)+"\\"+str(user)+" was not found")
            return
        user = "" if user is None else user
        domain = "" if domain is None else domain
        command_option = command_option.replace("|username|", user)
        command_option = command_option.replace("|domain|", domain)
        command_option = command_option.replace("|password|", user_o["password"])
        command_option = command_option.replace("|share|",share_m["share"])
        command_option = command_option.replace("|filepath|",path.replace("\\\\", "\\"))
        just_path = path.replace("\\\\", "\\").replace(share_m["share"], "",1).replace("/", "\\")
        while just_path.startswith("\\"):
            just_path = just_path[1:]
        command_option = command_option.replace("|file|",path.replace("\\\\", "\\").replace(share_m["share"], "",1))
        command_option = command_option.replace("|filename|",os.path.basename(path.replace("\\\\", "\\").replace(share_m["share"], "",1)))

        command_option = command_option.replace("|ip|",ip)
        self.tkApp.launch_in_terminal(None, "Share command", command_option, use_pollex=False)

    def userCommand(self, command_option, user):
        if user is None:
            selection_users = self.tvUsers.selection()
            if len(selection_users) >= 1:
                item = self.tvUsers.item(selection_users[0])
                user = (item["values"][1], item["text"], item["values"][0])
            else:
                user = None
        searching = [r"ask_text:([^:\|]+)", "computers_as_file"]
        for keyword in searching:
            s = re.search(r"\|"+keyword+r"\|", command_option)
            if s is not None:
                if keyword == "computers_as_file":
                    filepath = self.exportAllComputersAsFile()
                    command_option = command_option.replace(s.group(0), filepath)
                elif "|ask_text:" in s.group(0):
                    what = s.group(1)
                    dialog = ChildDialogAskText(self.tkApp, what)
                    self.tkApp.wait_window(dialog.app)
                    fp = tempfile.mkdtemp()
                    filepath = os.path.join(fp, what+".txt")
                    with open(filepath, mode="w") as f:
                        f.write(dialog.rvalue)
                    command_option = command_option.replace(s.group(0), filepath)
            command_option = command_option.replace("|domain|", user[0])
            command_option = command_option.replace("|username|", user[1])
            command_option = command_option.replace("|password|", user[2])
        self.tkApp.launch_in_terminal(None, "user command", command_option)

    def computerCommand(self, command_option, ips=None, user=None, treeview=None, model=None):
        if ips is None:
            ips = []
            selection = self.tvComputers.selection()
            for selected in selection:
                item = self.tvComputers.item(selected)
                ip = item["text"]
                ips.append(ip)
        if user is None:
            selection_users = self.tvUsers.selection()
            if len(selection_users) >= 1:
                item = self.tvUsers.item(selection_users[0])
                user = (item["values"][1], item["text"], item["values"][0])
            else:
                user = None
        for ip in ips:
            searching = ["wordlist", r"ask_text:([^:\|]+)", "users_as_file", "ip"]
            for keyword in searching:
                s = re.search(r"\|"+keyword+r"\|", command_option)
                if s is not None:
                    if keyword == "wordlist":
                        dialog = ChildDialogAskFile(self.tkApp, f"Choose a wordlist file")
                        command_option = command_option.replace(s.group(0), dialog.rvalue)
                    elif keyword == "ip":
                        command_option = command_option.replace(s.group(0), ip)
                    elif keyword == "users_as_file":
                        if user is not None and len(user) == 3:
                            domain = user[0]
                        else:
                            domain = ""
                        filepath = self.exportAllUsersAsFile(domain)
                        command_option = command_option.replace(s.group(0), filepath)
                    elif "|ask_text:" in s.group(0):
                        what = s.group(1)
                        dialog = ChildDialogAskText(self.tkApp, what)
                        self.tkApp.wait_window(dialog.app)
                        fp = tempfile.mkdtemp()
                        filepath = os.path.join(fp, what+".txt")
                        with open(filepath, mode="w") as f:
                            f.write(dialog.rvalue)
                        command_option = command_option.replace(s.group(0), filepath)
            user_searching = {"domain":None, "username":None, "password":None}
            if any(map(lambda user_keyword: f"|{user_keyword}|" in command_option, user_searching)):
                if user is not None and len(user) == 3:
                    command_option = command_option.replace("|domain|", user[0])
                    command_option = command_option.replace("|username|", user[1])
                    command_option = command_option.replace("|password|", user[2])
                else:
                    for user_keyword in user_searching:
                        if f"|{user_keyword}|" in command_option:
                            dialog = ChildDialogAskText(self.parent, "Enter "+str(user_keyword), multiline=False)
                            self.parent.wait_window(dialog.app)
                            if dialog.rvalue is None:
                                return
                            command_option = command_option.replace(f"|{user_keyword}|", dialog.rvalue)
            self.tkApp.launch_in_terminal(None, "computer command", command_option)
             
    def ask_confirmation(self, title, question):
        dialog = ChildDialogQuestion(self.parent, title, question)
        self.parent.wait_window(dialog.app)
        return dialog.rvalue == "Yes"
             
    def _insertChildren(self, coll_name, parent_data):
        """Insert every children defect in database as DefectView under this node"""
        if coll_name == "ips":
            apiclient = APIClient.getInstance()
            computers = apiclient.find("computers", {"ip": parent_data.get("ip")})
            for computer in computers:
                computer_o = ComputerController(Computer(computer))
                computer_vw = ComputerView(
                    self.tkApp.treevw, self.tkApp.viewframe, self.tkApp, computer_o)
                computer_vw.addInTreeview(str(parent_data.get("_id")), addChildren=False)
            shares = apiclient.find("shares", {"ip": parent_data.get("ip")})
            for share in shares:
                share_o = ShareController(Share(share))
                share_vw = ShareView(
                    self.tkApp.treevw, self.tkApp.viewframe, self.tkApp, share_o)
                share_vw.addInTreeview(str(parent_data.get("_id")), addChildren=False)
            return computers

class ChildDialogAddUsers:
    def __init__(self, parent, displayMsg="Add AD users"):
        """
        Open a child dialog of a tkinter application to ask a combobox option.

        Args:
            parent: the tkinter parent view to use for this window construction.
            displayMsg: The message that will explain to the user what he is choosing.
            default: Choose a default selected option (one of the string in options). default is None
        """
        self.app = CTkToplevel(parent, fg_color=utils.getBackgroundColor())
        self.app.attributes("-type", "dialog")
        self.app.resizable(False, False)
        appFrame = CTkFrame(self.app)
        self.app.title(displayMsg)
        self.rvalue = None
        self.parent = parent
        
        panel = FormPanel()
        self.formtext = panel.addFormText("Users", r"^\S+\\\S+:\S+$", default="domain\\username:password", side="top")
        button_panel = panel.addFormPanel(side="bottom")
        self.save_image = CTkImage(Image.open(utils.getIcon("save.png")))
        button_panel.addFormButton("Submit", self.onOk, side="right", image=self.save_image)
        b = button_panel.addFormButton("Cancel", self.onError, side="right", 
                               fg_color=utils.getBackgroundColor(), text_color=utils.getTextColor(),
                               border_width=1, border_color="firebrick1", hover_color="tomato")
        panel.constructView(appFrame)
        appFrame.pack(ipadx=10, ipady=5)
        try:
            self.app.wait_visibility()
            self.app.transient(parent)
            self.app.grab_set()
            self.app.focus_force()
            self.app.lift()
        except tk.TclError:
            pass

    def onOk(self, event=""):
        """he rvalue attributes to the value selected and close the window.
        """
        # send the data to the parent
        self.rvalue = self.formtext.getValue().split("\n")
        self.app.destroy()

    def onError(self, event=None):
        """
        Close the dialog and set rvalue to None
        """
        self.rvalue = None
        self.app.destroy()

class ChildDialogShare:
    def __init__(self, parent, share_data):
        self.app = CTkToplevel(parent, fg_color=utils.getBackgroundColor())
        self.app.attributes("-type", "dialog")
        self.app.resizable(True, True)
        appFrame = CTkFrame(self.app)
        self.app.title("View share info")
        self.rvalue = None
        self.parent = parent
        panel = FormPanel()
        panel_info = panel.addFormPanel(grid=True)
        panel_info.addFormLabel("IP")
        panel_info.addFormStr("IP", "", share_data.get("ip", ""), status="readonly", row=0, column=1)
        panel_info.addFormLabel("Share", row=1)
        panel_info.addFormStr("Share", "", share_data.get("share", ""), status="readonly", row=1, column=1)
        files = share_data.get('files', []) 
        if files is None:
            files = []
        defaults = []
        for file in files:
            defaults.append([file["path"],file["flagged"], file["size"]])
        panel.addFormTreevw("Files", ("Path","Flagged","Size"), default_values=defaults, side="top")
        button_panel = panel.addFormPanel(side="bottom")
        button_panel.addFormButton("Quit", self.onError, side="right")
        panel.constructView(appFrame)
        appFrame.pack(ipadx=10, ipady=5, expand=1)
        try:
            self.app.wait_visibility()
            self.app.transient(parent)
            self.app.grab_set()
            self.app.focus_force()
            self.app.lift()
        except tk.TclError:
            pass

    def onError(self, event=None):
        """
        Close the dialog and set rvalue to None
        """
        self.rvalue = None
        self.app.destroy()

class ChildDialogConfigureADModule:
    def __init__(self, parent, displayMsg="Configure AD module"):
        """
        Open a child dialog of a tkinter application to ask a combobox option.

        Args:
            parent: the tkinter parent view to use for this window construction.
            displayMsg: The message that will explain to the user what he is choosing.
            default: Choose a default selected option (one of the string in options). default is None
        """
        self.app = CTkToplevel(parent, fg_color=utils.getBackgroundColor())
        self.app.attributes("-type", "dialog")
        self.app.resizable(True , True)
        appFrame = CTkFrame(self.app)
        self.app.title(displayMsg)
        self.rvalue = None
        self.parent = parent
        self.save_image = CTkImage(Image.open(utils.getIcon("save.png")))
        panel = FormPanel()
        ad_settings = ActiveDirectory.getSettings()
        self.explanationLbl = panel.addFormText("Explanation","","""
        Explanations
        You can use those commands with different Types and Variables:
        
        Types:
            - Computer: those commands will be available when right clicking one or many computer and will execute the selected command for each one of them. 
                        It takes into account what user is selected in the user treeview or will ask information otherwise.
            - Share: those commands  will be available when right clicking one File in a share.
        Variables:
            - |ip|: will be replaced by the selected computer IP
            - |share|: (Only for Share type commands) Will be replaced with the share name (drive name like ADMIN$) selected in the share table.
            - |filepath|: (Only for Share type commands) Will be replaced with the full filepath selected in the share table (like ADMIN$\\folder\path.txt).
            - |file|: (Only for Share type commands) Will be replaced with the filepath WITHOUT THE SHARE NAME selected in the share table (like folder\path.txt).
            - |filename|: (Only for Share type commands) Will be replaced with only the filename selected in the share table (like path.txt).
            - |username|, |domain|, |password|: will be replaced by the selected user information, or will be prompted if nothing is selected.
            - |wordlist|: will be prompted for a wordlist type of file and replaced by the filepath given
            - |ask_text:$name|: prompt for a text where name is what is asked. store it in a file and replace in command by filepath
            - |users_as_file|: will be replaced with a filename containing all users usernames
            - |computers_as_file|: will be replaced with a filename containing all computers IPs
        """, state="disabled")
        self.formtv = panel.addFormTreevw("Commands", ("Type", "Command name", "Command line"), 
                            ad_settings.get("commands", []), doubleClickBinds=[["Computer", "Share"], "", ""],
                            side="top", width=800,height=10, fill=tk.BOTH)
        button_panel = panel.addFormPanel(side="bottom")
        button_panel.addFormButton("Submit", self.onOk, side="right",  image=self.save_image)
        button_panel.addFormButton("Cancel", self.onError, side="right", 
                               fg_color=utils.getBackgroundColor(), text_color=utils.getTextColor(),
                               border_width=1, border_color="firebrick1", hover_color="tomato")
        panel.constructView(appFrame)
        appFrame.pack(ipadx=10, ipady=5, fill=tk.BOTH, expand=tk.TRUE)
        try:
            self.app.wait_visibility()
            self.app.transient(parent)
            self.app.grab_set()
            self.app.focus_force()
            self.app.lift()
        except tk.TclError:
            pass

    def onOk(self, event=""):
        """he rvalue attributes to the value selected and close the window.
        """
        # send the data to the parent
        self.rvalue = self.formtv.getValue()
        self.app.destroy()

    def onError(self, event=None):
        """
        Close the dialog and set rvalue to None
        """
        self.rvalue = None
        self.app.destroy()


"""Tool Model. A tool is an instanciation of a command against a target"""

from pollenisatorgui.core.components.datamanager import DataManager
from pollenisatorgui.core.models.element import Element
from pollenisatorgui.core.components.apiclient import APIClient
from bson.objectid import ObjectId
from datetime import datetime
import pollenisatorgui.core.components.utils as utils
import re

class Tool(Element):
    """
    Represents a Tool object that defines a tool. A tool is a command run materialized on a runnable object (wave, scope, ip, or port)

    Attributes:
        coll_name: collection name in pollenisator database
    """
    coll_name = "tools"

    def __init__(self, valuesFromDb=None):
        """Constructor
        Args:
            valueFromDb: a dict holding values to load into the object. A mongo fetched interval is optimal.
                        possible keys with default values are : _id(None), parent(None), tags([]), infos({}),
                        command_iid(""), wave(""),  name(""), scope(""), ip(""), port(""), proto("tcp"), lvl(""), text(""), dated("None"),
                        datef("None"), scanner_ip("None"), status([]), notes(""), resultfile(""), plugin_used("")
        """
        if valuesFromDb is None:
            valuesFromDb = {}
        super().__init__(valuesFromDb.get("_id", None), valuesFromDb.get("parent", None), valuesFromDb.get("infos", {}))
        self.datef = "None"
        self.dated = "None"
        self.scanner_ip = "None"
        self.resultfile = ""
        self.plugin_used = ""
        self.status = []
        self.initialize(valuesFromDb.get("command_iid", ""), valuesFromDb.get("check_iid", ""), valuesFromDb.get("wave", ""),
                        valuesFromDb.get("name", ""),
                        valuesFromDb.get(
                            "scope", ""), valuesFromDb.get("ip", ""),
                        str(valuesFromDb.get("port", "")), valuesFromDb.get(
                            "proto", "tcp"),
                        valuesFromDb.get(
                            "lvl", ""), valuesFromDb.get("text", ""),
                        valuesFromDb.get("dated", "None"), valuesFromDb.get(
                            "datef", "None"),
                        valuesFromDb.get(
                            "scanner_ip", "None"), valuesFromDb.get("status", []), valuesFromDb.get("notes", ""), valuesFromDb.get("resultfile", ""), valuesFromDb.get("plugin_used", ""),  valuesFromDb.get("infos", {}))

    def initialize(self, command_iid, check_iid=None, wave="", name="", scope="", ip="", port="", proto="tcp", lvl="", text="",
                   dated="None", datef="None", scanner_ip="None", status=None, notes="", resultfile="", plugin_used="",  infos=None):
        
        """Set values of tool
        Args:
            command_iid: command associated for the tool 
            name: name of the tool (should match a command name)
            wave: the target wave name of this tool (only if lvl is "wave"). Default  ""
            scope: the scope string of the target scope of this tool (only if lvl is "port:onServiceUpdate"). Default  ""
            ip: the target ip "ip" of this tool (only if lvl is "ip" or "port"). Default  ""
            port: the target port "port number" of this tool (only if lvl is "port"). Default  ""
            proto: the target port "proto" of this tool (only if lvl is "port"). Default  "tcp"
            lvl: the tool level of exploitation (wave, network, ip ort port/). Default ""
            text: the command to be launched. Can be empty if name is matching  a command. Default ""
            dated: a starting date and tiem for this interval in format : '%d/%m/%Y %H:%M:%S'. or the string "None"
            datef: an ending date and tiem for this interval in format : '%d/%m/%Y %H:%M:%S'. or the string "None"
            scanner_ip: the worker name that performed this tool. "None" if not performed yet. Default is "None"
            status: a list of status string describing this tool state. Default is None. (Accepted values for string in list are done, running, OOT, OOS)
            notes: notes concerning this tool (opt). Default to ""
            resultfile: an output file generated by the tool. Default is ""
            plugin_used: plugin used when tool was imported
            tags: a list of tags
            infos: a dictionnary of additional info
        Returns:
            this object
        """
        self.command_iid = command_iid
        self.check_iid = None if check_iid is None else str(check_iid)
        self.name = name
        self.wave = wave
        self.scope = scope
        self.ip = ip
        self.port = str(port)
        self.proto = proto
        self.lvl = lvl
        self.text = text
        self.dated = dated
        self.datef = datef
        self.scanner_ip = scanner_ip
        self.notes = notes
        self.resultfile = resultfile
        self.plugin_used = plugin_used
        self.infos = infos if infos is not None else {}
        if status is None:
            status = []
        elif isinstance(status, str):
            status = [status]
        self.status = status
        return self

    def delete(self):
        """
        Delete the tool represented by this model in database.
        """
        apiclient = APIClient.getInstance()
        apiclient.delete("tools", ObjectId(self._id))

    def addInDb(self):
        """
        Add this tool in database.

        Returns: a tuple with :
                * bool for success
                * mongo ObjectId : already existing object if duplicate, create object id otherwise 
        """
        base = self.getDbKey()
        apiclient = APIClient.getInstance()
        datamanager = DataManager.getInstance()
        # Checking unicity
        existing = datamanager.find("tools", base, False)
        if existing is not None:
            return False, existing["_id"]
        # Those are added to base after tool's unicity verification
        base["name"] = self.name
        base["ip"] = self.ip
        base["scope"] = self.scope
        base["port"] = self.port
        base["proto"] = self.proto
        base["command_iid"] = self.command_iid
        base["check_iid"] = self.check_iid
        base["scanner_ip"] = self.scanner_ip
        base["dated"] = self.dated
        base["datef"] = self.datef
        if isinstance(self.status, str):
            self.status = [self.status]
        base["status"] = self.status
        base["text"] = self.text
        base["resultfile"] = self.resultfile
        base["plugin_used"] = self.plugin_used
        base["notes"] = self.notes
        base["infos"] = self.infos
        res, iid = apiclient.insert("tools", base)
        self._id = iid
        return True, iid
      
    def getData(self):
        """Return scope attributes as a dictionnary matching Mongo stored scopes
        Returns:
            dict with keys name, wave, scope, ip, port, proto, lvl, text, dated, datef, scanner_ip, notes, status, _id, tags and infos
        """

        return {"name": self.name, "command_iid":self.command_iid, "check_iid": self.check_iid, "wave": self.wave, "scope": self.scope,
                "ip": self.ip, "port": self.port, "proto": self.proto,
                "lvl": self.lvl, "text": self.text, "dated": self.dated,
                "datef": self.datef, "scanner_ip": self.scanner_ip,
                "notes": self.notes, "_id": self.getId(), "infos": self.infos, "status":self.getStatus()}

    def setStatus(self,status):
        """Set this tool status with given list of status
        Args:
            list of string with status inside (accepted values are OOS, OOT, running, done)
        """
        self.status = status
        apiclient = APIClient.getInstance()
        apiclient.setToolStatus(self.getId(), self.status)

    def getStatus(self):
        """
        Get the tool executing status.

        Return:
            Returns a list of status status are :
                OOT : Out of time = This tool is in a wave which does not have any interval for now.
                OOS : Out os scope = This tool is in an IP OOS
                done : This tool is completed
                running : This tool is being run."""
        return self.status

    def getCommand(self):
        """
        Get the tool associated command.

        Return:
            Returns the Mongo dict command fetched instance associated with this tool's name.
        """
        apiclient = APIClient.getInstance()
        if self.command_iid == "":
            return None
        commandTemplate = apiclient.findInDb(apiclient.getCurrentPentest(),
                                                 "commands", {"_id": ObjectId(self.command_iid)}, False)
        return commandTemplate

    @classmethod
    def __sanitize(cls, var_to_path):
        """Replace unwanted chars in variable given: '/', ' ' and ':' are changed to '_'
        Args:
            var_to_path: a string to sanitize to use a path folder
        Returns:
            modified arg as string
        """
        var_to_path = var_to_path.replace("/", "_")
        var_to_path = var_to_path.replace(" ", "_")
        var_to_path = var_to_path.replace(":", "_")
        return var_to_path

    def getOutputDir(self, pentestName):
        """
        Get the tool required output directory path.
        Args:
            pentestName: the pentest database name
        Return:
            Returns the output directory of this tool instance.
        """
        # get command needed directory
        output_dir = Tool.__sanitize(
            pentestName)+"/"+Tool.__sanitize(self.name)+"/"
        if self.wave != "" and self.wave is not None:
            output_dir += Tool.__sanitize(self.wave)+"/"
        if self.scope != "" and self.scope is not None:
            output_dir += Tool.__sanitize(self.scope)+"/"
        if self.ip != "" and self.ip is not None:
            output_dir += Tool.__sanitize(self.ip)+"/"
        if self.port != "" and self.port is not None:
            port_dir = str(self.port) if str(self.proto) == "tcp" else str(
                self.proto)+"/"+str(self.port)
            output_dir += Tool.__sanitize(port_dir)+"/"
        return output_dir

    

    def update(self, pipeline_set=None):
        """Update this object in database.
        Args:
            pipeline_set: (Opt.) A dictionnary with custom values. If None (default) use model attributes.
        """
        apiclient = APIClient.getInstance()
        if pipeline_set is None:
            apiclient.update("tools", ObjectId(self._id), {"scanner_ip": str(self.scanner_ip), "dated": str(self.dated), "status": self.status,
                         "datef":  str(self.datef), "notes":  self.notes, "resultfile": self.resultfile, "plugin_used":self.plugin_used, "infos":self.infos})
        else:
            apiclient.update(
                "tools", ObjectId(self._id), pipeline_set)

    def _getParentId(self):
        """
        Return the mongo ObjectId _id of the first parent of this object. For a Tool it is either a scope, an ip or a port depending on the tool's level.

        Returns:
            Returns the parent's ObjectId _id". or None if a type error occurs
        """
        try:
            if self.check_iid == "":
                datamanager = DataManager.getInstance()
                wave_o = datamanager.find("waves", {"wave": self.wave}, False)
                if wave_o is not None:
                    return str(wave_o.getId())
            elif self.check_iid is not None:
                return str(self.check_iid)
            return None
        except TypeError:
            # None type returned:
            return None

    def __str__(self):
        """
        Get a string representation of a tool.

        Returns:
            Returns the tool name. 
        """
        return self.name

    def getDetailedString(self):
        """
        Get a more detailed string representation of a tool.

        Returns:
            string
        """
        apiclient = APIClient.getInstance() 
        return apiclient.getToolDetailedString(self.getId())
        
    def getResultFile(self):
        """Returns the result file of this tool
        Returns:
            strings
        """
        return self.resultfile

    

    def markAsRunning(self, workerName, infos):
        """Set this tool status as running but keeps OOT or OOS.
        Sets the starting date to current time and ending date to "None"
        Args:
            workerName: the worker name that is running this tool
        """
        self.dated = datetime.now().strftime('%d/%m/%Y %H:%M:%S')
        self.datef = "None"
        newStatus = ["running"]
        if "OOS" in self.status:
            newStatus.append("OOS")
        if "OOT" in self.status:
            newStatus.append("OOT")
        self.status = newStatus
        self.scanner_ip = workerName
        if infos is None:
            infos = {}
        self.infos.update(infos)
        self.update()

    def markAsNotDone(self):
        """Set this tool status as not done by removing "done" or "running" status.
        Also resets starting and ending date as well as worker name
        """
        self.dated = "None"
        self.datef = "None"
        self.scanner_ip = "None"
        if "done" in self.status:
            self.status.remove("done")
        if "running" in self.status:
            self.status.remove("running")
        if "error" in self.status:
            self.status.remove("error")
        if not self.status:
            self.status = ["ready"]
        self.update()

    

    def getDbKey(self):
        """Return a dict from model to use as unique composed key.
        Returns:
        {"wave": self.wave, "lvl": self.lvl, "check_iid":self.check_iid}
        """
        base = {"wave": self.wave, "name":self.name, "lvl": self.lvl, "check_iid":self.check_iid}
        return base



"""Hold functions to interact with the admin api"""
from pollenisatorgui.core.components.apiclient import APIClient
from pollenisatorgui.core.application.dialogs.ChildDialogEditPassword import ChildDialogEditPassword
import tkinter as tk
import tkinter.ttk as ttk
from customtkinter import *
from PIL import Image, ImageTk
import pollenisatorgui.core.components.utils as utils
from pollenisatorgui.core.application.pollenisatorentry import PopoEntry


class AdminView:
    """View for admin module"""

    def __init__(self, nbk):
        self.nbk = nbk
        self.parent = None
        self.userTv = None

    def initUI(self, parent):
        """Create widgets and initialize them
        Args:
            parent: the parent tkinter widget container."""
        if self.userTv is not None:
            self.refreshUI()
            return
        self.parent = parent
        ### WORKER TREEVIEW : Which worker knows which commands
        self.userTv = ttk.Treeview(self.parent)
        self.headings = ["Username", "Password", "Admin", "Name", "Surname", "Email"]
        self.userTv['columns'] = self.headings
        self.userTv.heading("#0", text='Username', anchor="nw")
        self.userTv.column("#0", anchor="nw")
        self.userTv.heading("#1", text='Password', anchor="nw")
        self.userTv.column("#1", anchor="nw")
        self.userTv.heading("#2", text='Admin', anchor="nw")
        self.userTv.column("#2", anchor="nw")
        self.userTv.heading("#3", text='Name', anchor="nw")
        self.userTv.column("#3", anchor="nw")
        self.userTv.heading("#4", text='Surname', anchor="nw")
        self.userTv.column("#4", anchor="nw")
        self.userTv.heading("#5", text='Email', anchor="nw")
        self.userTv.column("#5", anchor="nw")
        self.userTv.pack(side=tk.TOP, padx=10, pady=10, fill=tk.X)
        self.userTv.bind("<Double-Button-1>", self.OnUserDoubleClick)
        self.userTv.bind("<Delete>", self.OnUserDelete)
        
        #### BUTTONS FOR AUTO SCANNING ####
        lblAddUsername = ttk.LabelFrame(parent, text="Add user", padding=10)
        addUserFrame = CTkFrame(lblAddUsername)
        lblAddUser = CTkLabel(addUserFrame, text="Username")
        lblAddUser.grid(column=0, sticky=tk.E)
        self.entryAddUser = PopoEntry(addUserFrame)
        self.entryAddUser.grid(row=0, column=1, sticky=tk.W)
        lblAddPwd = CTkLabel(addUserFrame, text="Password")
        lblAddPwd.grid(row=1, column=0, sticky=tk.E)
        self.password = tk.StringVar() 
        entryAddPwd = PopoEntry(addUserFrame, show="*", textvariable=self.password)
        entryAddPwd.grid(row=1, column=1, sticky=tk.W)
        lblAddConfirmPwd = CTkLabel(addUserFrame, text="Confirm")
        lblAddConfirmPwd.grid(row=2, column=0, sticky=tk.E)
        self.confirmpassword = tk.StringVar() 
        entryAddConfirmPwd = PopoEntry(addUserFrame, show="*", textvariable=self.confirmpassword)
        entryAddConfirmPwd.grid(row=2, column=1, sticky=tk.W)
        lblName = CTkLabel(addUserFrame, text="Name")
        lblName.grid(row=3, column=0, sticky=tk.E)
        self.name = PopoEntry(addUserFrame)
        self.name.grid(row=3, column=1, sticky=tk.W)
        lblSurname = CTkLabel(addUserFrame, text="Surname")
        lblSurname.grid(row=4, column=0, sticky=tk.E)
        self.surname = PopoEntry(addUserFrame)
        self.surname.grid(row=4, column=1, sticky=tk.W)
        lblEmail = CTkLabel(addUserFrame, text="Email")
        lblEmail.grid(row=5, column=0, sticky=tk.E)
        self.email = PopoEntry(addUserFrame)
        self.email.grid(row=5, column=1, sticky=tk.W)
        self.add_user_icon = CTkImage(Image.open(utils.getIcon("add_user.png")))
        btn_addUser = CTkButton(
                addUserFrame, text="Add user", image=self.add_user_icon, command=self.addUser)
        btn_addUser.grid(row=6, column = 2, sticky=tk.W)
        addUserFrame.pack()
        lblAddUsername.pack()

    def refreshUI(self):
        apiclient = APIClient.getInstance()
        users = apiclient.getUsers()
        for children in self.userTv.get_children():
            self.userTv.delete(children)
        for user in users:
            username = user["username"]
            name = user.get("name", "")
            surname = user.get("surname", "")
            email = user.get("email", "")
            admin = "Admin" if "admin" in user["scope"] else ""
            try:
                user_node = self.userTv.insert(
                    '', 'end', username, text=username, values=("Change pass", admin, name, surname, email))
            except tk.TclError:
                pass
       
    def addUser(self):
        apiclient = APIClient.getInstance()
        if self.confirmpassword.get() != self.password.get():
            tk.messagebox.showerror("Add user failed", "The password does not match the confirmation")
        username = self.entryAddUser.get()
        passw = self.password.get()
        name = self.name.get()
        surname = self.surname.get()
        email = self.email.get()
        apiclient.registerUser(username, passw, name, surname, email)
        self.userTv.insert('', 'end', username, text=username, values=("Change pass", '', name, surname, email,))

    def OnUserDoubleClick(self, event):
        treevw = event.widget
        item = treevw.identify("item", event.x, event.y)
        username = str(item)
        column = treevw.identify_column(event.x)
        columnNb = int(column[1:])
        values = treevw.item(item)["values"]
        if columnNb == 0:
            pass
        elif columnNb == 1:
            self.openEditPassword(username)
        elif columnNb > 1:
            oldVal = values[columnNb-1]
            newVal = tk.simpledialog.askstring(
                "Modify infos", "New value for "+self.headings[columnNb].lower(), initialvalue=oldVal)
            if newVal is None:
                return
            if newVal.strip() == "" or newVal.strip() == oldVal.strip():
                return
            newVals = list(values)
            newVals[columnNb-1] = newVal.strip()
            treevw.item(item, values=newVals)
            values = treevw.item(item)["values"]
            newName = values[self.headings.index("Name") - 1]
            newSurname = values[self.headings.index("Surname") - 1]
            newEMail = values[self.headings.index("Email") - 1]
            apiclient = APIClient.getInstance()
            apiclient.updateUserInfos(username, newName, newSurname, newEMail)

    def openEditPassword(self, username):
        dialog = ChildDialogEditPassword(self.parent, username, askOldPwd=False)
        self.parent.wait_window(dialog.app)
        self.refreshUI()
    
    def OnUserDelete(self, event):
        apiclient = APIClient.getInstance()
        username = self.userTv.selection()[0]
        apiclient.deleteUser(username) 
        self.userTv.delete(username)



# """Controllers for summary view of notebook"""
# import tkinter.ttk as ttk
# from customtkinter import *
# import tkinter as tk
# from pollenisatorgui.core.application.dialogs.ChildDialogProgress import ChildDialogProgress
# from pollenisatorgui.core.application.scrollableframexplateform import ScrollableFrameXPlateform
# from pollenisatorgui.core.models.ip import Ip
# from pollenisatorgui.core.models.port import Port
# from pollenisatorgui.core.components.settings import Settings
# from pollenisatorgui.core.components.apiclient import APIClient
# from pollenisatorgui.modules.module import Module

# def smart_grid(parent, root, *args, **kwargs):  # *args are the widgets!
#     """Grid adapted to each treeview size.
#     Adapted from stackoverflow but cannot find it anymore =/"""
#     divisions = kwargs.pop('divisions', 100)
#     force_f = kwargs.pop('force', False)
#     if 'sticky' not in kwargs:
#         kwargs.update(sticky='w')
#     try:
#         parent.win_width
#     except AttributeError:
#         parent.win_width = -1
#     parent.update_idletasks()
#     root.update_idletasks()
#     winfo_width = root.winfo_reqwidth()
#     if 1 < winfo_width != parent.win_width or force_f:
#         parent.win_width = winfo_width
#         row = col = width = 0
#         argc = len(args)

#         for i in range(argc):
#             widget_width = args[i].winfo_reqwidth()
#             columns = max(
#                 1, int(widget_width * float(divisions) / winfo_width))
#             width += widget_width
#             if width > winfo_width:
#                 width = widget_width
#                 row += 1
#                 col = 0
#             args[i].grid(row=row, column=col, columnspan=columns, **kwargs)
#             col += columns
#         parent.update_idletasks()  # update() #
#         return row + 1


# class Summary(Module):
#     """
#     Store elements to summarize the ongoing pentest
#     """
#     iconName = "tab_summary.png"
#     tabName = "  Summary  "
#     order_priority = Module.MEDIUM_PRIORITY
#     def __init__(self, root, settings):
#         """Constructor
#         Args:
#             root: the root widget of tkinter
#         """
#         super().__init__()
#         self.linkTw = None
#         self.root = root
#         self.treeviews = {}
#         self.parent = None
#         self.nbk = None
#         self.summaryFrame = None
#         self.scroller = None
#         self.frameTw = None
    
#     def open(self):
#         apiclient = APIClient.getInstance()
#         if apiclient.getCurrentPentest() is not None:
#             self.refreshUI()
#         return True

#     def refreshUI(self):
#         """Refresh information then reloads the view with them.
#         """
#         if self.frameTw is not None:
#             for widget in self.frameTw.winfo_children():
#                 widget.destroy()
#         if self.summaryFrame is not None:
#             for widget in self.summaryFrame.winfo_children():
#                 widget.destroy()
#         if self.treeviews is not None:
#             for k in list(self.treeviews.keys()):
#                 del self.treeviews[k]
#             del self.treeviews
#         self.treeviews = {}
#         self.frameTw = CTkFrame(self.summaryFrame)
#         self.frameTw.pack(side=tk.LEFT, padx=10, pady=10)
#         self.loadSummary()

#     def initUI(self, parent, nbk, linkTw, tkApp):
#         """Initialize widgets of the summary
#         Args:
#             parent: parent tkinter container widget 
#             nbk: a ref to the notebook
#             linkTw: the treeview holding more info in the notebook to be displayed after an interaction
#         """
#         self.parent = parent
#         self.nbk = nbk
#         if self.linkTw is not None:  # UI Already built
#             self.linkTw = linkTw
#             self.refreshUI()
#             return
#         self.linkTw = linkTw
#         self.summaryFrame = ScrollableFrameXPlateform(self.parent)
#         self.summaryFrame.pack(side=tk.TOP, anchor=tk.W, expand=True,
#                                fill=tk.BOTH, padx=10, pady=10)

#     def loadSummary(self):
#         """Reload information about IP and Port and reload the view.
#         """
#         apiclient = APIClient.getInstance()
#         nonEmptyIps = list(apiclient.aggregate("ports",[{"$group":{"_id":"$ip"}}, {"$count": "total"}]))
#         if not nonEmptyIps:
#             return
#         nonEmptyIps = nonEmptyIps[0]
#         step = 0
#         dialog = ChildDialogProgress(self.parent, "Loading summary ", "Refreshing summary. Please wait for a few seconds.", 200, "determinate")
#         dialog.show(nonEmptyIps["total"])
#         nonEmptyIps = apiclient.aggregate("ports",[{"$group":{"_id":"$ip"}}])
#         for ipCIDR in nonEmptyIps:
#             step += 1
#             ip = Ip.fetchObject({"ip": ipCIDR["_id"]})
#             if ip is None:
#                 continue
#             if ip.in_scopes:
#                 try:
#                     dialog.update(step)
#                 except tk.TkError as e:
#                     #probably claused the windows, stopping
#                     break
#                 self.insertIp(ip.ip)
#         self.frameTw.update_idletasks()
#         self.parent.update_idletasks()
#         smart_grid(self.frameTw, self.root, *list(self.treeviews.values()))
#         dialog.destroy()


#     def OnDoubleClick(self, event):
#         """Callback for treeview double click.
#         If a link treeview is defined, open mainview and focus on the item with same iid clicked.
#         Args:
#             event: used to identified which link was clicked. Auto filled
#         """
#         if self.linkTw is not None:
#             self.nbk.select("Main View")
#             tv = event.widget
#             item = tv.identify("item", event.x, event.y)
#             self.linkTw.focus(item)
#             self.linkTw.see(item)
#             self.linkTw.selection_set(item)

#     def updatePort(self, port_data):
#         """Update a port line according to a new port_data (just change colors now)
#         Args:
#             port_data: a port data dictionnary
#         """
#         newTags = port_data.get("tags", [])
#         treeviewToUpdate = self.treeviews.get(port_data["ip"], None)
#         if treeviewToUpdate is not None:
#             if port_data.get("_id", None) is not None:
#                 try:
#                     treeviewToUpdate.item(str(port_data["_id"]), tags=newTags)
#                 except:
#                     pass

#     def insertPort(self, port_o):
#         """Insert a new port in the summary
#         Args:
#             port_o: a port object to be inserted
#         """
#         treeviewToUpdate = self.treeviews.get(port_o["ip"], None)
#         if treeviewToUpdate is None:
#             self.insertIp(port_o["ip"])
#         treeviewToUpdate = self.treeviews.get(port_o["ip"], None)
#         port_text = port_o["port"]
#         if port_o["proto"] == "udp":
#             port_text = "udp/"+port_text
#         treeviewToUpdate.configure(height=len(treeviewToUpdate.get_children())+1)
#         treeviewToUpdate.insert('', 'end', str(
#             port_o["_id"]), text=port_text, tags=list(port_o["tags"]))

#     def insertIp(self, ip):
#         """Insert a new IP in the summary. Also insert its port
#         Args:
#             ip: an IP object to be inserted
#         """
#         treevw = ttk.Treeview(self.frameTw)
#         treevw.heading("#0", text=ip, anchor='w')
#         treevw.column("#0", anchor='w')
#         tags = Settings.getTags()
#         for tag, color in tags.items():
#             if color == "transparent": 
#                 continue
#             treevw.tag_configure(tag, background=color)
#         treevw.bind("<Double-Button-1>", self.OnDoubleClick)
#         count = 0
#         self.treeviews[ip] = treevw
#         ports = Port.fetchObjects({"ip": ip})
#         for port in ports:
#             if port.proto.strip() != "" and str(port.port).strip() != "":
#                 port_text = port.port
#                 if port.proto == "udp":
#                     port_text = "udp/"+port_text
#                 try:
#                     treevw.insert('', 'end', str(port.getId()),
#                               text=port_text, tags=list(port.getTags()))
#                 except:
#                     pass
#                 count += 1
#         treevw.configure(height=count)
#         treevw.update_idletasks()

#     def deleteIp(self, ip):
#         """Remvoe an IP from the summary.
#         Args:
#             ip: an IP object to be removed
#         """
#         if self.treeviews.get(ip, None) is not None:
#             del self.treeviews[ip]
